// $Id: ConnectionDialog.cpp,v 1.43 2012/12/08 20:12:01 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY;  without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.   See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA   02111-1307 USA
*
*******************************************************************************/

#include "ConnectionDialog.h"
#include "ConnectionFrame.h"
#include "ConnectionParser.h"
#include "CustomComboBox.h"
#include "CustomDialog.h"
#include "Icons.h"
#include "IconEngine.h"
#include "AnimatedLineEditor.h"
#include "QuestionDialog.h"
#include "Str.h"
#include "SimpleListView.h"
#include "XmlOptions.h"

#include <QtGui/QCheckBox>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QLayout>
#include <QtGui/QSplitter>

//_______________________________________________
const QString ConnectionDialog::Model::columnTitles_[ ConnectionDialog::Model::nColumns ] =
{
    "name",
    "Host",
    "Port",
    "User"
};

//______________________________________________________________________
ConnectionDialog::ConnectionDialog( QWidget* parent ):
    BaseDialog( parent ),
    Counter( "ConnectionDialog" )
{

    Debug::Throw( "ConnectionDialog::ConnectionDialog.\n" );

    setOptionName( "CONNECTION_DIALOG" );

    // vertical layout
    QVBoxLayout *layout( new QVBoxLayout() );
    layout->setMargin(10);
    layout->setSpacing(5);
    setLayout( layout );

    // horizontal layout
    QHBoxLayout* hLayout = new QHBoxLayout();
    hLayout->setMargin(0);
    hLayout->setSpacing(5);
    layout->addLayout( hLayout );

    // create list
    hLayout->addWidget( list_ = new SimpleListView( this ), 0 );
    list_->setModel( &model_ );

    // change font
    QFont boldFont( list_->font() );
    boldFont.setBold( true );
    list_->setFont( boldFont );

    // right widget
    QWidget* rightWidget;
    hLayout->addWidget( rightWidget = new QWidget( this ), 1 );

    // vertical layout
    QVBoxLayout* vLayout( new QVBoxLayout() );
    vLayout->setMargin(0);
    vLayout->setSpacing(5);
    rightWidget->setLayout( vLayout );

    // connection frame
    vLayout->addWidget( frame_ = new ConnectionFrame( rightWidget ) );

    // save password
    QCheckBox* checkbox = new QCheckBox( "Save passwords", rightWidget );
    vLayout->addWidget( checkbox );
    checkbox->setChecked( XmlOptions::get().get<bool>( "SAVE_PASSWORDS" ) );
    connect( checkbox, SIGNAL( toggled( bool ) ), SLOT( _toggleSavePasswords( bool ) ) );

    // connect
    vLayout->addWidget( connectButton_ = new QPushButton( "Connect", rightWidget ) );
    connect( connectButton_, SIGNAL( clicked() ), SLOT( _connect() ) );
    connect( connectButton_, SIGNAL( clicked() ), SLOT( close() ) );
    connectButton_->setToolTip( "Connect" );
    connectButton_->setDefault( true );
    connectButton_->setIcon( IconEngine::get( ICONS::CONNECT ) );

    // buttons
    vLayout->addWidget( saveButton_ = new QPushButton( "Save", rightWidget ) );
    connect( saveButton_, SIGNAL( clicked() ), SLOT( _save() ) );
    saveButton_->setToolTip( "Save current settings" );
    saveButton_->setIcon( IconEngine::get( ICONS::SAVE ) );

    vLayout->addWidget( removeButton_ = new QPushButton( "Remove", rightWidget ) );
    connect( removeButton_, SIGNAL( clicked() ), SLOT( _remove() ) );
    removeButton_->setToolTip( "Remove current settings from list" );
    removeButton_->setIcon( IconEngine::get( ICONS::REMOVE ) );

    QPushButton *cancelButton;
    vLayout->addWidget( cancelButton = new QPushButton( "Cancel", rightWidget ) );
    connect( cancelButton, SIGNAL( clicked() ), SLOT( _cancel() ) );
    connect( cancelButton, SIGNAL( clicked() ), SLOT( close() ) );
    cancelButton->setToolTip( "Discard changes and cancel connection" );
    cancelButton->setIcon( IconEngine::get( ICONS::DIALOG_CANCEL ) );

    connect( list_->selectionModel(), SIGNAL( selectionChanged(const QItemSelection&, const QItemSelection&) ), SLOT( _select() ) );
    connect( list_->selectionModel(), SIGNAL( selectionChanged(const QItemSelection&, const QItemSelection&) ), SLOT( _updateButtons() ) );

    // direct connection on activation.
    connect( list_, SIGNAL( activated(const QModelIndex&) ), SLOT( _select() ) );
    connect( list_, SIGNAL( activated(const QModelIndex&) ), SLOT( _connect() ) );
    connect( list_, SIGNAL( activated(const QModelIndex&) ), SLOT( close() ) );

    // load connection attributes into list
    _load();

    // update button state
    _updateButtons();
    adjustSize();
    Debug::Throw( "ConnectionDialog::ConnectionDialog - done.\n" );

}

//______________________________________________________________________
void ConnectionDialog::closeEvent( QCloseEvent* event )
{
    Debug::Throw( "ConnectionDialog::closeEvent.\n" );

    // save to resources
    Model::List connections( model_.get() );

    QModelIndex index( list_->selectionModel()->currentIndex() );
    if( index.isValid() )
    {
        for( Model::List::iterator iter = connections.begin(); iter != connections.end(); iter++ )
        { iter->setIsDefault( model_.index( *iter ).row() == index.row() ); }
    }

    // get current selection
    parser_.set( connections );
    parser_.write( QString( XmlOptions::get().raw( "RC_FILE" ) ) );

    return BaseDialog::closeEvent( event );
}

//______________________________________________________________________
void ConnectionDialog::_load( void )
{
    Debug::Throw( "ConnectionDialog::_load.\n" );

    // read from options
    parser_.read( QString( XmlOptions::get().raw( "RC_FILE" ) ) );

    // store in list
    model_.set( parser_.attributes() );

    // resize list
    list_->resizeColumnToContents( Model::NAME );
    list_->resizeColumnToContents( Model::USER );
    list_->resizeColumnToContents( Model::PORT );
    list_->resizeColumnToContents( Model::HOST );

    // select last item
    if( !parser_.attributes().empty() )
    {

        foreach( const ConnectionAttributes attributes, parser_.attributes() )
        {
            if( attributes.isDefault() )
            {

                // select default item
                QModelIndex index( model_.index( attributes ) );
                list_->selectionModel()->select( index, QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows );
                list_->selectionModel()->setCurrentIndex( index, QItemSelectionModel::Select|QItemSelectionModel::Rows );

                // assign to frame
                frame().load( attributes );
                break;
            }
        }


    }

}

//______________________________________________________________________
void ConnectionDialog::_select( void )
{
    Debug::Throw( "ConnectionDialog::_select.\n" );
    QModelIndexList indexes( list_->selectionModel()->selectedRows() );
    if( !indexes.empty() ) frame().load( model_.get(indexes.front()) );
}

//______________________________________________________________________
void ConnectionDialog::_remove( void )
{

    Debug::Throw( "ConnectionDialog::_Remove.\n" );

    // retrieve selection
    ConnectionAttributes::List selection( model_.get( list_->selectionModel()->selectedRows() ) );
    Q_ASSERT( selection.size() == 1 );

    // confirm
    QString buffer;
    QTextStream( &buffer ) << "Remove connection named " << selection.begin()->name() << " from list ?";
    if( !QuestionDialog( this, buffer ).exec() ) return;

    model_.remove( *selection.begin() );
    list_->resizeColumnToContents( Model::NAME );
    list_->resizeColumnToContents( Model::USER );
    list_->resizeColumnToContents( Model::PORT );
    list_->resizeColumnToContents( Model::HOST );

}

//______________________________________________________________________
void ConnectionDialog::_save( void )
{

    Debug::Throw( "ConnectionDialog::_save.\n" );
    CustomDialog dialog( this );
    CustomComboBox* combobox;

    dialog.mainLayout().addWidget( new QLabel( "Connection name:", &dialog ) );
    dialog.mainLayout().addWidget( combobox  = new CustomComboBox( &dialog ) );
    combobox->setEditable( true );
    combobox->setAutoCompletion( true, Qt::CaseSensitive );

    // get model, and data; populate combobox
    foreach( const ConnectionAttributes attributes, model_.get() )
    { combobox->addItem( attributes.name() ); }

    // select current item
    QModelIndex index( list_->selectionModel()->currentIndex() );
    if( index.isValid() )
    { combobox->setCurrentIndex( combobox->findText( model_.get( index ).name() ) ); }


    dialog.adjustSize();
    dialog.centerOnParent();

    if( dialog.exec() == QDialog::Rejected ) return;
    model_.add( frame().attributes().setName( combobox->currentText() ) );

}

//______________________________________________________________________
void ConnectionDialog::_connect( void )
{
    Debug::Throw( "ConnectionDialog::_connect.\n" );
    done( QDialog::Accepted );
}

//______________________________________________________________________
void ConnectionDialog::_cancel( void )
{
    Debug::Throw( "ConnectionDialog::_cancel.\n" );
    done( QDialog::Rejected );
}

//______________________________________________________________________
void ConnectionDialog::_toggleSavePasswords( bool state )
{
    Debug::Throw( "ConnectionDialog::_toggleSavePassword.\n" );
    XmlOptions::get().set<bool>( "SAVE_PASSWORDS", state );
}

//______________________________________________________________________
void ConnectionDialog::_updateButtons( void )
{
    Debug::Throw( "ConnectionDialog::_updateButtons.\n" );
    removeButton_->setEnabled( !list_->selectionModel()->selectedRows().empty() );
}

//______________________________________________________________________
ConnectionDialog::Model::Model( QObject* parent ):
    ListModel<ConnectionAttributes>( parent ),
    Counter( "ConnectionDialog::Model" ),
    icon_( IconEngine::get( ICONS::CONNECTION ) )
{}

//______________________________________________________________________
QVariant ConnectionDialog::Model::data( const QModelIndex& index, int role ) const
{
    Debug::Throw( "ConnectionDialog::Model::data.\n" );

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated file info
    const ConnectionAttributes& attributes( get()[index.row()] );

    // return text associated to file and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {
            case NAME: return attributes.name();
            case HOST: return attributes.host();
            case PORT: return Str().assign<unsigned int>( attributes.port() );
            case USER: return attributes.user();
            default: return QVariant();
        }

    } else if( role == Qt::DecorationRole && index.column() == NAME ) {

        return icon_;

    }

    return QVariant();

}

//______________________________________________________________________
QVariant ConnectionDialog::Model::headerData(int section, Qt::Orientation orientation, int role ) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}
