#ifndef FileInfo_h
#define FileInfo_h

// $Id: FileInfo.h,v 1.30 2012/12/10 21:59:12 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "BaseFileInfo.h"
#include "Counter.h"

#include <QtXml/QDomElement>
#include <QtXml/QDomDocument>

//! store a file and needed informations for Ftp transfer
class FileInfo: public BaseFileInfo, public Counter
{

    public:

    //! file info list
    typedef QListIterator<FileInfo> ListIterator;
    typedef QPair<FileInfo, FileInfo> Pair;
    typedef BaseFileInfo::BaseList<FileInfo> List;

    //! status
    enum Status
    {
        Overwrite = 1<<0,
        Conflict = 1<<1,
        Success = 1<<2
    };

    Q_DECLARE_FLAGS( StatusFlags, Status )

    //! constructor
    FileInfo( const QString& file = QString(), TypeFlags type = None ):
        BaseFileInfo( file, type ),
        Counter( "FileInfo" ),
        status_( Success | Overwrite )
    {}

    //! constructor from base class
    FileInfo( const BaseFileInfo& other ):
        BaseFileInfo( other ),
        Counter( "FileInfo" ),
        status_( Success | Overwrite )
    {}

    //! constructor from DOM element
    FileInfo( const QDomElement& );

    //! destructor
    virtual ~FileInfo( void )
    {}

    //! dump to dom element
    virtual QDomElement domElement( QDomDocument& ) const;

    //!@name accessors
    //@{

    //! status
    StatusFlags status( void ) const
    { return status_; }

    //! success
    bool isSuccess( void ) const
    { return status_&Success; }

    //! failed
    bool isFailed( void ) const
    { return !isSuccess(); }

    //! conflict
    bool isConflict( void ) const
    { return status_&Conflict; }

    //! overwrite
    bool isOverwrite( void ) const
    { return status_&Overwrite; }

    //! return path of the file.
    /*! properly deal with local/remote filesystems */
    File path( void ) const;

    //@}

    //!@name modifiers
    //@{

    //! flags
    FileInfo& setStatus( const Status& status, const bool& value )
    {
        if( value ) status_ |= status;
        else status_ &= (~status);
        return *this;
    }

    //! status
    FileInfo& setStatus( StatusFlags status )
    {
        status_ = status;
        return *this;
    }

    //! mark fileInfo as success
    FileInfo& setSuccess( void )
    { return setStatus( Success, true ); }

    //! mark fileInfo as failed
    FileInfo& setFailed( void )
    { return setStatus( Success, false ); }

    //@}

    //! used to see if non directory files have conflicts
    class ConflictFTor
    {

        public:

        //! predicate
        bool operator() (const FileInfo& info ) const
        { return info.isConflict() && info.isDocument(); }

    };

    //! used to count files with success set to false
    class FailureFTor
    {

        public:

        //! predicate
        bool operator() (const FileInfo& info ) const
        { return info.isFailed(); }

    };

    private:

    //! status
    StatusFlags status_;

};

Q_DECLARE_OPERATORS_FOR_FLAGS( FileInfo::StatusFlags)

#endif
