// $Id: FileInfoModel.cpp,v 1.53 2013/01/07 19:19:23 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "FileInfoModel.h"

#include "FileIconProvider.h"
#include "TimeStamp.h"
#include "XmlDef.h"
#include "XmlOptions.h"
#include "QOrderedSet.h"

#include <QtCore/QTimer>
#include <QtCore/QUrl>
#include <QtGui/QIcon>

//_______________________________________________________________
FileInfoModel::FileInfoModel( QObject* parent, BaseFileInfo::TypeFlags type ):
    BaseFileInfoModel<FileInfo>( parent ),
    Counter( "FileInfoModel" ),
    type_( type ),
    iconProvider_( 0x0 )
{  Debug::Throw( "FileInfoModel::FileInfoModel.\n" ); }

//__________________________________________________________________
Qt::ItemFlags FileInfoModel::flags(const QModelIndex &index) const
{

    Qt::ItemFlags flags( ListModel<FileInfo>::flags( index ) );
    if( !index.isValid() )
    {

        flags |= Qt::ItemIsDropEnabled;

    } else if( contains( index ) ) {

        // retrieve associated file info
        const FileInfo& fileInfo( get()[index.row()] );
        if( fileInfo.isNavigator() ) flags &= ~Qt::ItemIsSelectable;
        else {

            flags |= Qt::ItemIsDragEnabled;
            if( fileInfo.isFolder() ) flags |= Qt::ItemIsDropEnabled;

        }

    }

    return flags;

}

//__________________________________________________________________
QVariant FileInfoModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated file info
    const FileInfo& fileInfo( get()[index.row()] );

    // return text associated to file and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case PERMISSIONS:
            return (fileInfo.isRemote() || fileInfo.type() & FileInfo::Navigator) ? QString():fileInfo.permissionsString();

            case LOCATION:
            return QString( fileInfo.isLocal() ? "local":"remote");

            case STATUS:
            return QString( fileInfo.isSuccess() ? "success":"failed" );

            case ACTION:
            return QString( fileInfo.isOverwrite() ? "overwrite":"skip" );

            default:
            return BaseFileInfoModel<FileInfo>::data( index, role );
        }
    }

    return BaseFileInfoModel<FileInfo>::data( index, role );

}

//______________________________________________________________________
QStringList FileInfoModel::mimeTypes( void ) const
{
    QStringList types;
    if( _isRemote() ) types << BaseFileInfo::MimeType << "text/uri-list";
    if( _isLocal() ) types << BaseFileInfo::MimeType;
    return types;
}

//______________________________________________________________________
QMimeData* FileInfoModel::mimeData(const QModelIndexList &indexes) const
{

    QOrderedSet<QString> filenames;
    QOrderedSet<FileInfo> fileInfoList;

    foreach( const QModelIndex& index, indexes )
    {

        if( index.isValid() )
        {
            FileInfo info( get(index) );
            if( info.isNavigator() ) continue;

            fileInfoList.insert( info );
            filenames.insert( info.file() );
        }

    }

    if( filenames.empty() ) return 0;
    else {

        QMimeData *mimeData = new QMimeData();

        // fill text and url data
        QList<QUrl> urlList;
        QString fullText;
        QTextStream buffer( &fullText );
        foreach( const QString& filename, filenames )
        {
            QString url;
            QTextStream( &url ) << ( (type_ & FileInfo::Remote ) ? "ftp://" : "file://" ) << filename;
            buffer << url << endl;
            urlList << url;
        }

        mimeData->setText( fullText );
        if( _isLocal() ) mimeData->setUrls( urlList );

        // fill Drag data. Use XML
        QDomDocument document;
        QDomElement top = document.appendChild( document.createElement( XML::FILEINFO_LIST ) ).toElement();
        foreach( const FileInfo& fileInfo, fileInfoList )
        {

            if( fileInfo.file().isEmpty() ) continue;
            top.appendChild( fileInfo.domElement( document ) );

        }

        mimeData->setData( BaseFileInfo::MimeType, document.toByteArray() );
        return mimeData;

    }

}

//__________________________________________________________________
bool FileInfoModel::dropMimeData(const QMimeData* data , Qt::DropAction action, int, int, const QModelIndex& parent)
{

    // check action
    if( action == Qt::IgnoreAction) return true;

    // file info
    FileInfoModel::List fileInfoList;

    // Drag from Keyword model
    if( data->hasFormat( BaseFileInfo::MimeType ) )
    {

        // get dropped file info (use XML)
        // dom document
        QDomDocument document;
        if( !document.setContent( data->data( BaseFileInfo::MimeType ), false ) ) return false;

        QDomElement docElement = document.documentElement();
        QDomNode node = docElement.firstChild();
        for(QDomNode node = docElement.firstChild(); !node.isNull(); node = node.nextSibling() )
        {
            QDomElement element = node.toElement();
            if( element.isNull() ) continue;

            // special options
            if( element.tagName() == XML::FILEINFO )
            {

                const FileInfo fileInfo( element );
                if( fileInfo.file().isEmpty() || fileInfo.isNavigator() ) continue;

                fileInfoList << fileInfo;

            }
        }

    } else if( _isRemote() && data->hasUrls() ) {

        const QList<QUrl> urls( data->urls() );
        foreach( const QUrl& url, urls )
        {

            #if QT_VERSION >= 0x040800
            // check that local file
            if( !url.isLocalFile() ) continue;
            #endif

            // get file and check existence
            const File file( url.path() );
            if( !file.exists() ) continue;

            FileInfo fileInfo( file );
            fileInfo.update();

            fileInfo.setLocal();
            if( file.isDirectory() ) fileInfo.setIsFolder();
            else fileInfo.setIsDocument();
            if( file.isLink() ) fileInfo.setIsLink();
            if( file.isBrokenLink() ) fileInfo.setIsBrokenLink();
            if( file.isHidden() ) fileInfo.setIsHidden();

            fileInfoList << fileInfo;

        }

    }

    if( fileInfoList.empty() ) return false;

    // check destination
    FileInfo destination;
    if( parent.isValid() && get( parent ).isFolder() )
    {

        // get destination path, and add trailing "/" if missing
        destination = get( parent );
        if( !( destination.file().isEmpty() || destination.file().endsWith( "/" ) ) )
        { destination.setFile( destination.file() + "/" ); }

    } else destination.setType( type_ );

    // delayed transfer request emission
    // this fixes issues with nested event loops
    transferData_.set( destination, fileInfoList );
    QTimer::singleShot( 0, this, SLOT( _requestTransfer() ) );
    return true;

}

//__________________________________________________________________
QVariant FileInfoModel::headerData(int column, Qt::Orientation orientation, int role) const
{

    if( orientation == Qt::Horizontal && role == Qt::DisplayRole )
    {
        switch( column )
        {
            case LOCATION: return "Location";
            case STATUS: return "Status";
            case ACTION: return "Action";
            default: break;
        }
    }

    // base class implementation
    return BaseFileInfoModel<FileInfo>::headerData( column, orientation, role );

}

//____________________________________________________________
void FileInfoModel::_sort( int column, Qt::SortOrder order )
{ std::sort( _get().begin(), _get().end(), SortFTor( (ColumnType) column, order ) ); }

//____________________________________________________________
QIcon FileInfoModel::_icon( const QModelIndex& index ) const
{ return iconProvider_ ? iconProvider_->icon( get( index ) ):QIcon(); }

//________________________________________________________
bool FileInfoModel::SortFTor::operator () ( const FileInfo& constFirst, const FileInfo& constSecond ) const
{

    // Navigator come first
    // Folders come next
    if( constFirst.isNavigator() ) return true;
    else if( constSecond.isNavigator() ) return false;
    else if(
        (constFirst.type()&(BaseFileInfo::Folder|BaseFileInfo::Document)) !=
        (constSecond.type()&(BaseFileInfo::Folder|BaseFileInfo::Document)) )
    { return constFirst.isFolder(); }


    FileInfo first( constFirst );
    FileInfo second( constSecond );
    if( order_ == Qt::DescendingOrder ) std::swap( first, second );

    switch( type_ )
    {

        case LOCATION:
        {
            if( first.isLocal() && second.isRemote() ) return true;
            else if( second.isLocal() && first.isRemote() ) return false;
            else return first.file() < second.file();
        }

        case STATUS:
        return ( first.isSuccess() && !second.isSuccess() );

        case ACTION:
        return ( first.isOverwrite() && !second.isOverwrite() );

        default:
        return BaseFileInfoModel<FileInfo>::SortFTor::operator() ( constFirst, constSecond );

    }

}
