// $Id: FileSelectionFrame.cpp,v 1.100 2013/02/06 11:29:52 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "FileSelectionFrame.h"

#include "AnimatedTreeView.h"
#include "BaseContextMenu.h"
#include "Color.h"
#include "ColumnSelectionMenu.h"
#include "ColumnSortingMenu.h"
#include "CustomToolButton.h"
#include "Icons.h"
#include "IconSize.h"
#include "IconEngine.h"
#include "IconView.h"
#include "InformationDialog.h"
#include "NewFolderDialog.h"
#include "PathEditor.h"
#include "RemoveFilesDialog.h"
#include "RenameFileDialog.h"
#include "BaseFileInfoToolTipWidget.h"
#include "Util.h"

#include <QtGui/QLayout>
#include <QtCore/QDir>
#include <QtGui/QLineEdit>

//_____________________________________________
class FileInfoView: public IconView
{

    public:

    //! constructor
    FileInfoView( QWidget* widget = 0 ):
        IconView( widget )
        {}

    //! destructor
    virtual ~FileInfoView( void )
    {}

    protected:

    //! drag move
    virtual void dragMoveEvent( QDragMoveEvent* );

};

//_____________________________________________
void FileInfoView::dragMoveEvent( QDragMoveEvent* event )
{
    IconView::dragMoveEvent( event );
    FileInfoModel* model( 0 );
    if( _hoverIndex().isValid() && (model = qobject_cast<FileInfoModel*>( this->model() ) ) )
    {
        if( !model->get( _hoverIndex() ).isFolder() )
        { _setHoverIndex( QModelIndex() ); }
    }
}

//_____________________________________________
FileSelectionFrame::FileSelectionFrame( QWidget *parent, BaseFileInfo::TypeFlags type ):
    QWidget( parent ),
    Counter( "FileSelectionFrame" ),
    model_( this, type ),
    selectionModel_( &model_, this ),
    list_( 0x0 ),
    columnSelectionMenu_( 0x0 ),
    columnSortingMenu_( 0x0 ),
    type_( type ),
    visibilityOptionName_( _isLocal() ? "Local_VIEW_VISIBILITY":"Remote_VIEW_VISIBILITY" ),
    viewModeOptionName_( _isLocal() ? "LOCAL_VIEW_MODE":"REMOTE_VIEW_MODE" ),
    viewMode_( ICONS ),
    showHiddenFiles_( false ),
    active_( true )
{

    QVBoxLayout *layout = new QVBoxLayout();
    layout->setSpacing(0);
    layout->setMargin(0);
    setLayout( layout );

    // path editor
    pathEditor_ = new PathEditor( this );
    pathEditor_->setDragEnabled( true );
    pathEditor_->setIsLocal( _isLocal() );
    pathEditor_->setPrefix( _isLocal() ? "file:":"ftp:" );
    layout->addWidget( pathEditor_ );

    // tooltip manager
    toolTipWidget_ = new BaseFileInfoToolTipWidget( this );

    // install actions
    _installActions();

    // list
    viewMode_ = (FileSelectionFrame::ViewMode) XmlOptions::get().get<unsigned int>( viewModeOptionName_ );
    _setupList();

    connect( pathEditor_, SIGNAL( pathChanged( const File& ) ), SLOT( _reload( void ) ) );
    connect( pathEditor_, SIGNAL( pathChanged( const File& ) ), SLOT( _updateNavigationActions( void ) ) );

    // update buttons
    _updateNavigationActions();
    _updateActions();

    // visibility
    if( XmlOptions::get().contains( visibilityOptionName_ ) )
    { visibilityAction().setChecked( XmlOptions::get().get<bool>( visibilityOptionName_ ) ); }

}

//_________________________________________________________
void FileSelectionFrame::setHomePath( const File& path ) const
{ pathEditor_->setHomePath( path ); }

//_________________________________________________________
void FileSelectionFrame::setRootPathList( const File::List& value ) const
{ pathEditor_->setRootPathList( value ); }

//_________________________________________________________
void FileSelectionFrame::setPath( const File& path )
{
    Debug::Throw( "FileSelectionFrame::setPath.\n" );
    pathEditor_->setPath( path );
    _updateNavigationActions();
}

//________________________________________
FileInfo FileSelectionFrame::path( void ) const
{ return FileInfo( pathEditor_->path(), type_ | FileInfo::Folder ); }

//________________________________________
void FileSelectionFrame::clear( void )
{
    pathEditor_->setPath( File("/") );
    model_.clear();
}

//_________________________________________________________
void FileSelectionFrame::add( const FileInfo& fileInfo, unsigned int flags )
{
    Debug::Throw() << "FileSelectionFrame::add - file: " << fileInfo << endl;

    // check hidden files
    if( !_showHiddenFiles() && fileInfo.file().isHidden() ) return;

    // check that path match
    const File localPath( fileInfo.path() );
    if( QDir( localPath ) !=  QDir( path().file() ) )
    {
        Debug::Throw() << "FileSelectionFrame::add - invalid path: " << localPath << " vs " << path().file() << endl;
        return;
    }

    // retrieve and check local file
    const QString localName( fileInfo.file().localName() );
    if( localName.isEmpty() || localName == "." || localName == ".." ) return;

    // add to model
    model_.add( fileInfo );

    // retrieve matching index
    const QModelIndex index( model_.index( fileInfo ) );
    if( flags&VISIBLE ) list_->scrollTo( index );
    if( flags&SELECTED ) list_->selectionModel()->select( index, QItemSelectionModel::SelectCurrent|QItemSelectionModel::Rows );

}

//_________________________________________________________
void FileSelectionFrame::remove( const FileInfo& fileInfo )
{
    Debug::Throw() << "FileSelectionFrame::remove - file: " << fileInfo << endl;

    // retrieve and check local file
    const QString localName( fileInfo.file().localName() );
    if( localName.isEmpty() || localName == "." || localName == ".." ) return;

    // remove from model
    model_.remove( fileInfo );

    return;

}

//______________________________________________________
FileInfo::List FileSelectionFrame::selection( void ) const
{ return model_.get( list_->selectionModel()->selectedRows() ); }

//______________________________________________________
bool FileSelectionFrame::owns( QWidget* widget ) const
{
    // parse parent hierarchy
    for(;widget; widget = widget->parentWidget() )
    { if( widget == this ) return true; }
    return false;
}

//______________________________________________________
bool FileSelectionFrame::setViewMode( ViewMode value )
{

    if( viewMode_ == value ) return false;
    Debug::Throw( "FileSelectionFrame::setViewMode.\n" );

    viewMode_ = value;
    XmlOptions::get().set<unsigned int>( viewModeOptionName_, viewMode_ );
    _setupList();

    return true;

}

//________________________________________
void FileSelectionFrame::clearSelection( void )
{ if( list_ ) list_->selectionModel()->clear(); }

//________________________________________
void FileSelectionFrame::select( const FileInfo& info )
{

    if( !list_ ) return;

    const QModelIndex index( model_.index( info ) );
    if( index.isValid() )
    {
        list_->selectionModel()->select( index, QItemSelectionModel::Select|QItemSelectionModel::Rows );
        list_->selectionModel()->setCurrentIndex( index, QItemSelectionModel::Current );
    }

}

//______________________________________________________
void FileSelectionFrame::_indexActivated( const QModelIndex& index )
{
    Debug::Throw( "FileSelectionFrame::_indexActivated.\n" );
    FileInfo fileInfo( model_.get( index ) );

    if( fileInfo.isNavigator() ) parentDirectoryAction().trigger();
    else emit fileSelected( fileInfo );

}

//______________________________________________________
void FileSelectionFrame::_reload( void )
{
    Debug::Throw( "FileSelectionFrame::_reload.\n" );
    if( !path().file().isEmpty() ) emit fileSelected( path() );
}

//______________________________________________________
void FileSelectionFrame::_newFolder( void )
{
    Debug::Throw( "FileSelectionFrame::_newFolder.\n" );

    // retrieve current path
    const FileInfo path( this->path() );

    // dialog
    NewFolderDialog dialog( this, path );
    dialog.setWindowTitle( "Create New Folder - qftp" );
    if( !dialog.exec() ) return;

    // destination
    FileInfo destination( File( dialog.editor().text() ).addPath( path.file() ),  FileInfo::Folder|type() );

    // generate macros and submit
    FtpMacro::List macros;
    macros << ( FtpMacro( FtpMacro::CreateDirectory ) << path << destination );
    macros << FtpMacro::Clear;
    macros << ( FtpMacro( FtpMacro::ClearSelection ) << path );
    macros << ( FtpMacro( FtpMacro::ShowDirectory ) << path );
    macros << ( FtpMacro( FtpMacro::Select ) << destination );

    emit macrosRequested( macros );
}

//______________________________________________________
void FileSelectionFrame::_homeDirectory( void )
{
    Debug::Throw( "FileSelectionFrame::_homeDirectory.\n" );
    emit fileSelected( FileInfo( _isLocal() ? Util::home():File("/"), type_ | FileInfo::Folder ) );
}

//________________________________________
void FileSelectionFrame::_cut( void )
{
    Debug::Throw( "FileSelectionFrame::_cut.\n" );
    const FileInfoModel::List selection( this->selection() );
    if( !selection.empty() ) emit clipboardChanged( FileSelectionClipboard( selection, FileSelectionClipboard::Cut ) );
}

//________________________________________
void FileSelectionFrame::_copy( void )
{
    Debug::Throw( "FileSelectionFrame::_copy.\n" );
    const FileInfoModel::List selection( this->selection() );
    if( !selection.empty() ) emit clipboardChanged( FileSelectionClipboard( selection, FileSelectionClipboard::Copy ) );
}

//________________________________________
void FileSelectionFrame::_paste( void )
{ emit pasteClipboardRequested( path() ); }

//________________________________________
void FileSelectionFrame::_pasteInFolder( void )
{

    Debug::Throw( "FileSelectionFrame::_pasteInFolder.\n" );
    const FileInfo::List selection( this->selection() );
    Q_ASSERT( selection.size() == 1 && selection.front().isFolder() );
    emit pasteClipboardRequested( selection.front() );

}

//________________________________________
void FileSelectionFrame::_rename( void )
{
    Debug::Throw( "FileSelectionFrame::_rename.\n" );

    // path
    const FileInfo path( this->path() );

    // retrieve selected files
    const FileInfoModel::List selection( this->selection() );
    if( selection.size() != 1 ) return;

    FileInfo source( selection.front() );
    RenameFileDialog dialog( this, source );
    dialog.setWindowTitle( "Rename Item - Qftp" );
    if( !dialog.exec() ) return;

    FileInfo destination = FileInfo( source );
    destination.setFile( dialog.editor().text().trimmed() );
    if( destination.file().isEmpty() || destination.file() == source.file().localName() ) return;
    destination.setFile( destination.file().addPath( source.path() ) );

    // TODO: check if destination already exists
    FtpMacro::List macros;
    macros << ( FtpMacro( FtpMacro::Rename ) << source << destination );
    macros << ( FtpMacro( FtpMacro::ShowDirectory ) << path );
    macros << ( FtpMacro( FtpMacro::ClearSelection ) << path );
    macros << ( FtpMacro( FtpMacro::Select ) << destination );

    emit macrosRequested( macros );

}

//________________________________________
void FileSelectionFrame::_remove( void )
{
    Debug::Throw( "FileSelectionFrame::_remove.\n" );

    // retrieve selection
    const FileInfoModel::List selection( this->selection() );
    if( selection.empty() ) return;

    // ask confirmation
    RemoveFilesDialog dialog( this, selection );
    dialog.setWindowTitle( "Delete Files - Gridftp" );
    if( !dialog.exec() ) return;

    FtpMacro::List macros;

    macros << FtpMacro::Clear;
    foreach( const FileInfo& fileInfo, selection )
    {
        if( fileInfo.isRemote() ) macros << ( FtpMacro( FtpMacro::ListRecursive ) << fileInfo );
        else macros << ( FtpMacro( FtpMacro::Add ) << fileInfo );
    }

    // emit update macro
    macros
        << ( FtpMacro( FtpMacro::Remove ) << path() )
        << FtpMacro::Clear
        << ( FtpMacro( FtpMacro::ShowDirectory ) << path() );

    emit macrosRequested( macros );
}

//______________________________________________________
void FileSelectionFrame::_open( void )
{
    Debug::Throw( "FileSelectionFrame::_open.\n" );

    // create temporary list
    bool hasFolders( false );
    FileInfo::List local;
    foreach( FileInfo fileInfo, selection() )
    {
        // check fileInfo type
        if( fileInfo.isFolder() ) hasFolders = true;
        else local << fileInfo;
    }

    // check
    if( hasFolders ) { InformationDialog( this, "Warning: folders cannot be opened \nusing external applications." ).exec(); }
    else if( local.empty() ) { InformationDialog( this, "No valid file selected. <Open> canceled." ).exec(); }
    if( local.empty() ) return;

    // emit signal
    emit openFilesRequested( local );

}

//______________________________________________________
void FileSelectionFrame::_fileProperties( void )
{

    // get selected index
    const QModelIndex index( list_->selectionModel()->selectedRows().front() );

    // get file info
    const FileInfo fileInfo( model_.get( index ) );

    // icon
    QIcon icon;
    QVariant iconVariant( model_.data( index, Qt::DecorationRole ) );
    if( iconVariant.canConvert( QVariant::Icon ) ) icon = iconVariant.value<QIcon>();

    emit requestFileInformation( fileInfo, icon );

}

//______________________________________________________
void FileSelectionFrame::_updateContextMenu( const QPoint& position )
{
    Debug::Throw( "FileSelectionFrame::_updateContextMenu.\n" );

    BaseContextMenu menu( list_ );
    menu.setHideDisabledActions( true );

    TreeView* treeView( 0 );
    if( ( treeView = qobject_cast<TreeView*>( list_ ) ) )
    {

        menu.addAction( &treeView->showHeaderAction() );
        if( !treeView->showHeaderAction().isChecked() )
        {
            menu.addMenu( columnSelectionMenu_ );
            menu.addMenu( columnSortingMenu_ );
        }

    } else {

        menu.addMenu( columnSortingMenu_ );

    }

    // new folder
    menu.addSeparator();
    menu.addAction( &newFolderAction() );

    // cut/copy/paste command
    menu.addSeparator();
    menu.addAction( &cutAction() );
    menu.addAction( &copyAction() );


    const FileInfo::List selection( this->selection() );
    if( selection.size() == 1 && selection.front().isFolder() && pasteInFolderAction().isEnabled() )
    {

        menu.addAction( &pasteInFolderAction() );

    } else if( pasteAction().isEnabled() ) {

        menu.addAction( &pasteAction() );

    }

    // rename/remove commands
    menu.addSeparator();
    menu.addAction( &renameAction() );
    menu.addAction( &removeAction() );

    // open action
    menu.addSeparator();
    menu.addAction( &openAction() );
    menu.addAction( &filePropertiesAction() );

    // execute
    menu.exec( list_->viewport()->mapToGlobal( position ) );

}

//______________________________________________________
void FileSelectionFrame::_updateActions( void )
{
    Debug::Throw( "FileSelectionFrame::_updateActions.\n" );
    FileInfo::List selection( this->selection() );
    const bool hasSelection( !selection.empty() );

    // copy and remove actions
    if( hasSelection )
    {

        cutAction().setEnabled( true );
        cutAction().setText( QString( "Cut " ) + selection.description( FileInfo::List::Designate ) );

        copyAction().setEnabled( true );
        copyAction().setText( QString( "Copy " ) + selection.description( FileInfo::List::Designate ) );

        removeAction().setEnabled( true );
        removeAction().setText( QString( "Delete " ) + selection.description( FileInfo::List::Designate ) );

    } else {

        cutAction().setEnabled( false );
        cutAction().setText( "Cut" );

        copyAction().setEnabled( false );
        copyAction().setText( "Copy" );

        removeAction().setEnabled( false );
        removeAction().setText( "Delete" );

    }

    // rename action
    if( selection.size() == 1 )
    {

        renameAction().setEnabled( true );
        renameAction().setText( QString( "Rename " ) + selection.description( FileInfo::List::Designate ) + "..." );

    } else {

        renameAction().setEnabled( false );
        renameAction().setText( "Rename..." );

    }

    // file properties
    filePropertiesAction().setEnabled( selection.size() == 1 );

    // open action
    selection.erase( std::remove_if( selection.begin(), selection.end(), FileInfo::IsFolderFTor() ), selection.end() );
    if( !selection.empty() )
    {

        openAction().setEnabled( true );
        openAction().setText( QString( "Open " ) + selection.description( FileInfo::List::Designate ) + "..." );

    } else {

        openAction().setEnabled( false );
        openAction().setText( "Open..." );

    }

}

//__________________________________________________________
void FileSelectionFrame::_updateBackgroundColors( void ) const
{

    // adjust background colors for relevant widgets
    BASE::Color base( palette().color(  QPalette::Active, QPalette::Base ) );
    BASE::Color inactiveBase( palette().color(  QPalette::Inactive, QPalette::Base ) );
    if( !active_ )
    {
        base = base.merge( palette().color( QPalette::Active, QPalette::Window ), 0.6 );
        inactiveBase = inactiveBase.merge( palette().color( QPalette::Inactive, QPalette::Window ), 0.6 );
    }

    QList<QWidget*> widgets;
    widgets << list_ << pathEditor_;
    foreach( QWidget* widget, widgets )
    {
        QPalette palette( widget->palette() );
        palette.setColor( QPalette::Active, QPalette::Base, base );
        palette.setColor( QPalette::Inactive, QPalette::Base, inactiveBase );
        widget->setPalette( palette );
    }

    return;

}

//______________________________________________________
void FileSelectionFrame::_setupList( void )
{
    Debug::Throw( "FileSelectionFrame::_setupList.\n" );

    // keep track of focus
    const bool hasFocus( list_ && list_->hasFocus() );

    FileInfo::List selection;

    // cleanup
    if( list_ )
    {
        // store selection
        selection = model_.get( list_->selectionModel()->selectedRows() );

        // delete list
        list_->setParent( 0x0 );
        list_->deleteLater();
    }

    // create new list
    if( viewMode_ == ICONS ) list_ = new FileInfoView( this );
    else list_ = new AnimatedTreeView( this );

    layout()->addWidget( list_ );
    list_->setContextMenuPolicy( Qt::CustomContextMenu );
    list_->setModel( &model_ );
    list_->setSelectionModel( &selectionModel_ );
    list_->setSelectionMode( QAbstractItemView::ContiguousSelection );
    list_->setDragEnabled( true );
    list_->setAcceptDrops( true );
    list_->setDropIndicatorShown( true );
    list_->setMouseTracking( true );

    // option name
    const QString optionName( _isLocal() ? "LOCAL_LIST":"REMOTE_LIST" );
    if( TreeView* treeView = qobject_cast<TreeView*>(list_) )
    {

        treeView->setItemMargin( 2 );

        // lock columns
        foreach( const int& column, QList<int>() << FileInfoModel::LOCATION << FileInfoModel::STATUS << FileInfoModel::ACTION )
        {
            treeView->hideColumn( column );
            treeView->lockColumnVisibility( column );
        }

        treeView->setColumnHidden( FileInfoModel::FILE, false );
        treeView->lockColumnVisibility( FileInfoModel::FILE );

        // assign option
        treeView->setOptionName( optionName );

    }  else static_cast<IconView*>(list_)->setOptionName( optionName );

    // recreate menus
    if( columnSelectionMenu_ ) columnSelectionMenu_->deleteLater();
    if( columnSortingMenu_ ) columnSortingMenu_->deleteLater();

    if( QTreeView* treeView = qobject_cast<QTreeView*>( list_ ) )
    {

        columnSelectionMenu_ = new ColumnSelectionMenu( this, treeView );
        columnSortingMenu_ = new ColumnSortingMenu( this, treeView->header() );

    } else if( IconView* iconView = qobject_cast<IconView*>( list_ ) ) {

        columnSelectionMenu_ = 0x0;
        columnSortingMenu_ = new ColumnSortingMenu( this, iconView->header() );

    } else {

        columnSelectionMenu_ = 0x0;
        columnSortingMenu_ = 0x0;

    }

    // connections
    connect( list_->selectionModel(), SIGNAL( selectionChanged(const QItemSelection &, const QItemSelection &) ), SLOT( _updateActions() ) );
    connect( list_, SIGNAL( activated( const QModelIndex& ) ), SLOT( _indexActivated( const QModelIndex& ) ) );
    connect( list_, SIGNAL( hovered( const QModelIndex& ) ), SLOT( _showToolTip( const QModelIndex& ) ) );
    connect( list_, SIGNAL( customContextMenuRequested( const QPoint& ) ), SLOT( _updateContextMenu( const QPoint& ) ) );

    // actions
    list_->addAction( &newFolderAction() );
    list_->addAction( &cutAction() );
    list_->addAction( &copyAction() );
    list_->addAction( &pasteAction() );
    list_->addAction( &pasteInFolderAction() );
    list_->addAction( &renameAction() );
    list_->addAction( &removeAction() );
    list_->addAction( &openAction() );
    list_->addAction( &filePropertiesAction() );

    // restore focus
    if( hasFocus ) list_->setFocus();

    // restore selection
    if( !selection.isEmpty() )
    {
        list_->selectionModel()->clear();
        foreach( const FileInfo& fileInfo, selection )
        { list_->selectionModel()->select( model_.index( fileInfo ), QItemSelectionModel::Select|QItemSelectionModel::Rows ); }

    }

    // background
    _updateBackgroundColors();

    // adjust tooltip mode
    toolTipWidget_->setFollowMouse( viewMode_ == LIST );

}

//______________________________________________________
void FileSelectionFrame::_updateNavigationActions( void )
{
    Debug::Throw( "FileSelectionFrame::_updateNavigationActions.\n" );
    parentDirectoryAction().setEnabled( pathEditor_->hasParent() );
    previousDirectoryAction().setEnabled( pathEditor_->hasPrevious() );
    nextDirectoryAction().setEnabled( pathEditor_->hasNext() );
    if( isActive() ) emit actionsChanged();
    return;
}

//______________________________________________________
void FileSelectionFrame::_showToolTip( const QModelIndex& index )
{
    if( !index.isValid() ) toolTipWidget_->hide();
    else {

        // fileInfo
        const FileInfo fileInfo( model_.get( index ) );

        // icon
        QIcon icon;
        QVariant iconVariant( model_.data( index, Qt::DecorationRole ) );
        if( iconVariant.canConvert( QVariant::Icon ) ) icon = iconVariant.value<QIcon>();

        // rect
        QRect rect( list_->visualRect( index ) );
        rect.translate( list_->viewport()->mapToGlobal( QPoint( 0, 0 ) ) );
        toolTipWidget_->setIndexRect( rect );

        // assign to tooltip widget
        toolTipWidget_->setFileInfo( fileInfo, icon );

        // show
        toolTipWidget_->showDelayed();

    }

}

//_____________________________________________
void FileSelectionFrame::_installActions( void )
{

    Debug::Throw( "FileSelectionFrame::_installActions.\n" );

    // toggle visibility action
    addAction( visibilityAction_ = new QAction( _isLocal() ? "Show Local File System":"Show Remote File System", this ) );
    visibilityAction_->setCheckable( true );
    visibilityAction_->setChecked( true );
    connect( visibilityAction_, SIGNAL( toggled( bool ) ), SLOT( _toggleVisibility( bool ) ) );

    // reload
    addAction( reloadAction_ = new QAction( IconEngine::get( ICONS::RELOAD ), "Reload", this ) );
    connect( reloadAction_, SIGNAL( triggered( void ) ), SLOT( _reload( void ) ) );

    // previous directory
    addAction( previousDirectoryAction_ = new QAction( IconEngine::get( ICONS::PREVIOUS_DIRECTORY ), "Previous", this ) );
    connect( previousDirectoryAction_, SIGNAL( triggered( void ) ), pathEditor_, SLOT( selectPrevious( void ) ) );
    previousDirectoryAction_->setToolTip( "Change path to previous directory (from history)" );

    // next directory (from history)
    addAction( nextDirectoryAction_ = new QAction( IconEngine::get( ICONS::NEXT_DIRECTORY ), "Next", this ) );
    connect( nextDirectoryAction_, SIGNAL( triggered( void ) ), pathEditor_, SLOT( selectNext( void ) ) );
    nextDirectoryAction_->setToolTip( "Change path to next directory (from history)" );

    // parent directory in tree
    addAction( parentDirectoryAction_ = new QAction( IconEngine::get( ICONS::PARENT ), "Parent Directory", this ) );
    connect( parentDirectoryAction_, SIGNAL( triggered( void ) ), pathEditor_, SLOT( selectParent( void ) ) );
    parentDirectoryAction_->setToolTip( "Change path to parent directory" );

    // home directory
    addAction( homeDirectoryAction_ = new QAction( IconEngine::get( ICONS::HOME ), "Home", this ) );
    connect( homeDirectoryAction_, SIGNAL( triggered( void ) ), SLOT( _homeDirectory( void ) ) );
    homeDirectoryAction_->setToolTip( "Change path to user home" );

    // new folder
    newFolderAction_ = new QAction( IconEngine::get( ICONS::CREATE_DIRECTORY ), "Create New Folder...", this );
    connect( newFolderAction_, SIGNAL( triggered( void ) ), SLOT( _newFolder( void ) ) );

    // cut
    cutAction_ = new QAction( IconEngine::get( ICONS::CUT ), "Cut", this );
    cutAction_->setShortcut( QKeySequence::Cut );
    cutAction_->setShortcutContext(Qt::WidgetShortcut);
    connect( cutAction_, SIGNAL( triggered( void ) ), SLOT( _cut( void ) ) );

    // copy
    copyAction_ = new QAction( IconEngine::get( ICONS::COPY ), "Copy", this );
    copyAction_->setShortcut( QKeySequence::Copy );
    copyAction_->setShortcutContext(Qt::WidgetShortcut);
    connect( copyAction_, SIGNAL( triggered( void ) ), SLOT( _copy( void ) ) );

    // paste
    pasteAction_ = new QAction( IconEngine::get( ICONS::PASTE ), "Paste", this );
    pasteAction_->setShortcut( QKeySequence::Paste );
    pasteAction_->setShortcutContext(Qt::WidgetShortcut);
    connect( pasteAction_, SIGNAL( triggered( void ) ), SLOT( _paste( void ) ) );

    // paste in folder
    pasteInFolderAction_ = new QAction( IconEngine::get( ICONS::PASTE ), "Paste Into Folder", this );
    connect( pasteInFolderAction_, SIGNAL( triggered( void ) ), SLOT( _pasteInFolder( void ) ) );

    // rename
    renameAction_ = new QAction( IconEngine::get( ICONS::RENAME ), "Rename ...", this );
    renameAction_->setShortcut( Qt::Key_F2 );
    renameAction_->setShortcutContext(Qt::WidgetShortcut);
    connect( renameAction_, SIGNAL( triggered( void ) ), SLOT( _rename( void ) ) );

    // remove
    removeAction_ = new QAction( IconEngine::get( ICONS::DELETE ), "Delete", this );
    removeAction_->setShortcut( QKeySequence::Delete );
    removeAction_->setShortcutContext(Qt::WidgetShortcut);
    connect( removeAction_, SIGNAL( triggered( void ) ), SLOT( _remove( void ) ) );

    // open
    addAction( openAction_ = new QAction( IconEngine::get( ICONS::EDIT ), "Open...", this ) );
    connect( openAction_, SIGNAL( triggered( void ) ), SLOT( _open( void ) ) );

    // file properties
    filePropertiesAction_ = new QAction( IconEngine::get( ICONS::INFORMATION ), "Properties...", this );
    filePropertiesAction_->setShortcut( Qt::ALT + Qt::Key_Return );
    connect( filePropertiesAction_, SIGNAL( triggered( void ) ), SLOT( _fileProperties( void ) ) );

}
