#ifndef _FtpInterface_h_
#define _FtpInterface_h_

// $Id: FtpInterface.h,v 1.37 2012/10/31 18:19:11 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributped in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "FileThread.h"
#include "FtpCommand.h"
#include "FtpMacro.h"
#include "TextFormat.h"

#include <QtGui/QColor>
#include <QtGui/QCloseEvent>
#include <QtNetwork/QFtp>

/*!
\class FtpInterface
\brief customized QVBox to recieve close event
*/
class FtpInterface: public QObject, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor;
    FtpInterface( QObject* );

    //! clear
    void clear( void )
    {
        Debug::Throw( "FtpInterface::clear.\n" );
        macros_.clear();
        commands_.clear();
    }

    //! reference to ftp object
    QFtp& ftp( void )
    { return ftp_; }

    //!@name interface to store files
    //@{

    //! list of files
    const FileInfo::List& filesRecursive( void ) const
    { return filesRecursive_; }

    //! list of files
    FileInfo::List& filesRecursive( void )
    { return filesRecursive_; }

    //@}

    signals:

    //! directory content is available
    void contentAvailable( const FileInfo&, const FileInfo::List& );

    //! directory size
    void sizeAvailable( const FileInfo&, qint64 );

    //! emmited when some FileInfo has been updated
    void fileInfoUpdated( const FileInfo& );

    //! emmited when some FileInfo has been updated
    void fileInfoUpdated( const FileInfo::List& );

    //! file added
    void fileInfoAdded( const FileInfo& );

    //! file removed
    void fileInfoRemoved( const FileInfo& );

    //!@name macro interface
    //@{

    //! new macro added
    void macroAdded( const FtpMacro& );

    //! macro removed (prior to start)
    void macroRemoved( const FtpMacro& );

    //! new macro started
    void macroStarted( const FtpMacro& );

    //! macro completed
    void macroCompleted( const FtpMacro&, bool );

    //! ftp command added
    void commandAdded( const FtpCommand& );

    //! commands added
    void commandsAdded( const FtpCommand::List& );

    //! ftp command started
    void commandStarted( const FtpCommand& );

    //! ftp command completed
    void commandCompleted( const FtpCommand&, bool );

    //! command failed
    void commandFailed( const FtpCommand&, const QString& = QString() );

    //@}

    //! emitted for logging of processes
    void logAvailable( const QString&, FORMAT::TextFormatFlags = FORMAT::Default, QColor = QColor(), int = 0 );

    //! emmited when interface is busy
    void busy( void );

    //! emmited when interface is idle
    void ready( void );

    //! emmited when interface is busy
    void busyStateChanged( bool );

    //! emitted to show transfer progress
    void maxProgressAvailable( int );

    //! emitted to show transfer progress
    void progressAvailable( int );

    //! emitted when scratch file is created
    void scratchFileCreated( const File& );

    public slots:

    //! add ftp macro
    void addMacro( const FtpMacro& );

    //! add ftp macro
    void addMacros( const FtpMacro::List& );

    //! show hidden files
    void setShowHiddenFiles( bool );

    protected slots:

    //! run macro
    void _runMacro( void );

    //! current macro finished
    void _macroCompleted( bool = false );

    //! command finished
    void _commandStarted( int );

    //! command finished
    void _commandCompleted( int = 0, bool = false );

    //! transfer progress
    void _progress( qint64, qint64 );

    //! add files from remote listing
    void _addRemoteFile( const QUrlInfo& );

    //! process files from thread
    void _processFiles( const File::List& );

    //! process size from thread
    void _processSize( qint64 );

    protected:

    //!@name command interface
    //@{

    //! add command to queue
    void _addCommand( const FtpCommand& );

    //! add commands
    void _addCommands( const FtpCommand::List& );

    //! process command from queue
    void _runCommand( void );

    //! emmit message for given command and error status
    void _logCommand( const FtpCommand&, bool );

    //@}

    //! show hidden files
    const bool& _showHiddenFiles( void ) const
    { return showHiddenFiles_; }

    //! add file to recursive list
    void _addFileRecursive( const FileInfo& file )
    { filesRecursive_ << file; }

    //! clear list of files
    void _clearFilesRecursive( void )
    { filesRecursive_.clear(); }

    //! add file
    void _addFile( const FileInfo& fileInfo )
    { files_ << fileInfo; }

    //! clear files
    void _clearFiles( void )
    { files_.clear(); }

    //! remove path from a given file
    void _removePath( File& source, File path ) const;

    //! reset total and transfered size
    void _resetTotalSize( void );

    //! update total size from file_ content
    bool _updateTotalSize( void );

    //! compute total size and reset transfered size
    void _computeTotalSize( void );

    //! update transfered size
    void _updateTransferedSize( const FileInfo& file )
    { transfered_ += file.size(); }

    private:

    //!@name stored url informations for remote listing
    //@{

    //! list of files
    /*!
    filesRecursive_ is used to store all files read either remotely
    or locally, that can be retrieved internally
    */
    FileInfo::List filesRecursive_;

    //! list of files
    /*!
    files_ store all files read from a single "ls" command,
    they are copied into filesRecursives_ when several such commands
    must be called in a row
    */
    FileInfo::List files_;

    //@}

    //! show hidden files
    bool showHiddenFiles_;

    //! Ftp connection manager
    QFtp ftp_;

    //! file thread, for local files manipulations
    FileThread thread_;

    //! list of macros
    FtpMacro::List macros_;

    //! map command ids and types
    FtpCommand::List commands_;

    //! total size to transfer
    qint64 total_;

    //! transfered size
    qint64 transfered_;

};

#endif
