// $Id: FtpMacroModel.cpp,v 1.4 2012/11/22 22:25:33 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "FtpMacroModel.h"

#include "CustomPixmap.h"
#include "Icons.h"
#include "IconEngine.h"
#include "IconSize.h"
#include "TimeStamp.h"
#include "XmlOptions.h"

#include <QtGui/QIcon>

//_______________________________________________
const QString FtpMacroModel::columnTitles_[ FtpMacroModel::nColumns ] =
{ "Id", "Name", "File" };

//_______________________________________________________________
FtpMacroModel::FtpMacroModel( QObject* parent ):
    ListModel<FtpMacro>( parent ),
    Counter( "FtpMacroModel" )
{

    Debug::Throw( "FtpMacroModel::FtpMacroModel.\n" );
    updateConfiguration();

}

//__________________________________________________________________
QVariant FtpMacroModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated macro
    const FtpMacro& macro( get()[index.row()] );

    // return text associated to macro and column
    if( role == Qt::DisplayRole ) {

        switch( index.column() )
        {

            case ID: return macro.id();
            case NAME: return macro.name();
            case FILE: return macro.argumentCount() > 0  ? macro.argument(0).file():QVariant();
            default: return QVariant();
        }
    }

    // return icon associated to file
    if( role == Qt::DecorationRole && index.column() == NAME )
    { return _icon(macro.type()); }

    return QVariant();

}

//__________________________________________________________________
QVariant FtpMacroModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//_________________________________________________________
void FtpMacroModel::updateConfiguration( void )
{

    Debug::Throw( "FtpMacroModel::updateConfiguration.\n" );

    // install pixmaps
    _icons().clear();
    _installIcons();

    reset();

}

//_____________________________________________________________________
void FtpMacroModel::_installIcons( void ) const
{

    Debug::Throw( "FtpMacroList::_installIcons" );

    if( !_icons().empty() ) return;

    // type icons
    typedef QHash< FtpMacro::Type, QString > IconNames;
    IconNames typeNames;
    typeNames[FtpMacro::Connect] = ICONS::CONNECT;
    typeNames[FtpMacro::Add] = ICONS::ADD;
    typeNames[FtpMacro::Clear] = ICONS::DELETE;
    typeNames[FtpMacro::Abort] = ICONS::ABORT;
    typeNames[FtpMacro::CopyRecursive] = ICONS::PASTE;
    typeNames[FtpMacro::CreateDirectory] = ICONS::CREATE_DIRECTORY;
    typeNames[FtpMacro::Rename] = ICONS::RENAME;
    typeNames[FtpMacro::Remove] = ICONS::DELETE;
    typeNames[FtpMacro::Open] = ICONS::EDIT;
    typeNames[FtpMacro::ShowDirectory] = ICONS::FIND;
    typeNames[FtpMacro::ListRecursive] = ICONS::FIND;

    // load link pixmap
    for( IconNames::iterator iter = typeNames.begin(); iter != typeNames.end(); ++iter )
    { _icons().insert( iter.key(), IconEngine::get( iter.value() ) ); }

    return;
}

//_______________________________________________
FtpMacroModel::IconCache& FtpMacroModel::_icons( void )
{
    static IconCache cache;
    return cache;
}

//____________________________________________________________
const QIcon& FtpMacroModel::_icon( FtpMacro::Type type )
{

    IconCache::iterator iter = ( _icons().find( type ) );
    if( iter != _icons().end() ) return iter.value();
    else return _icons().insert( type, CustomPixmap().empty( IconSize( IconSize::Medium ) ) ).value();

}

//________________________________________________________
bool FtpMacroModel::SortFTor::operator () ( FtpMacro first, FtpMacro second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    switch( type_ )
    {

        case FtpMacroModel::NAME:
        if( first.type() != second.type() ) return first.type() < second.type();
        else break;

        case FtpMacroModel::FILE:
        {
            const FileInfo source( first.argumentCount() > 0 ? first.argument(0).file():FileInfo() );
            const FileInfo destination( second.argumentCount() > 0 ? second.argument(0).file():FileInfo() );
            if( source.file() != destination.file() ) return source.file() < destination.file();
            else break;
        }

        default: break;
    }

    return first.id() < second.id();

}
