// $Id: MainWindow.cpp,v 1.115 2013/01/08 18:39:19 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* software; if not, write to the Free Software Foundation, Inc., 59 Temple
* Place, Suite 330, Boston, MA  02111-1307 USA
*
*
*******************************************************************************/

#include "MainWindow.h"

#include "AnimatedTreeView.h"
#include "Application.h"
#include "CheckFilesDialog.h"
#include "ColumnSelectionMenu.h"
#include "ColumnSortingMenu.h"
#include "ConflictDialog.h"
#include "ConnectionDialog.h"
#include "ConnectionFrame.h"
#include "ContextMenu.h"
#include "CopyFilesDialog.h"
#include "CustomComboBox.h"
#include "CustomToolBar.h"
#include "DockWidget.h"
#include "FileIconProvider.h"
#include "FileInformationDialog.h"
#include "FileSelectionFrame.h"
#include "FtpCommandDialog.h"
#include "FtpInterface.h"
#include "Icons.h"
#include "IconEngine.h"
#include "InformationDialog.h"
#include "LogDialog.h"
#include "LogWidget.h"
#include "PlacesWidget.h"
#include "Menu.h"
#include "OpenFilesDialog.h"
#include "ProgressStatusBar.h"
#include "QuestionDialog.h"
#include "Singleton.h"
#include "Str.h"
#include "Util.h"
#include "ViewModeMenu.h"
#include "XmlOptions.h"

#include <QtCore/QDir>
#include <QtGui/QApplication>
#include <QtGui/QLayout>
#include <QtGui/QSplitter>

//____________________________________________________________________________________________
MainWindow::MainWindow( QWidget* parent ):
    BaseMainWindow( parent ),
    Counter( "MainWindow" ),
    fileSystemWatcher_( this ),
    interface_( new FtpInterface( this ) ),
    showNavigator_( true ),
    enabled_( true )
{

    Debug::Throw( "MainWindow::MainWindow.\n" );
    setOptionName( "MAIN_WINDOW" );

    // actions
    _installActions();
    Application& application( *Singleton::get().application<Application>() );
    addAction( &application.closeAction() );

    // main widget and layout
    QWidget* main = new QWidget( this );
    QVBoxLayout* mainLayout( new QVBoxLayout() );
    main->setLayout( mainLayout );
    mainLayout->setMargin(0);
    mainLayout->setSpacing(2);
    setCentralWidget( main );

    QHBoxLayout* hLayout( new QHBoxLayout() );
    hLayout->setMargin(0);
    hLayout->setSpacing(2);
    mainLayout->addLayout( hLayout );

    // splitter
    QSplitter *splitter;
    hLayout->addWidget( splitter = new QSplitter( main ) );

    // file selection frames
    splitter->addWidget( localSelectionFrame_ = new FileSelectionFrame( main, FileInfo::None ) );
    splitter->addWidget( remoteSelectionFrame_ = new FileSelectionFrame( main, FileInfo::Remote ) );

    // create icon provider and assign
    iconProvider_ = new FileIconProvider( this );
    foreach( FileSelectionFrame* frame, selectionFrames() )
    { frame->model().setIconProvider( iconProvider_ ); }

    //! dialogs
    commandDialog_ = new FtpCommandDialog();
    logDialog_ = new LogDialog();

    connect( commandDialog_, SIGNAL( dialogHidden() ), SLOT( _commandDialogHidden() ) );
    connect( logDialog_, SIGNAL( dialogHidden() ), SLOT( _logDialogHidden() ) );

    // toolbar
    CustomToolBar* toolbar;
    toolbar = new CustomToolBar( "Main Toolbar", this, "TRANSFER_TOOLBAR" );

    toolbar->addAction( &connectAction() );
    toolbar->addSeparator();
    toolbar->addAction( &homeDirectoryAction() );
    toolbar->addAction( &previousDirectoryAction() );
    toolbar->addAction( &nextDirectoryAction() );
    toolbar->addAction( &parentDirectoryAction() );
    toolbar->addSeparator();
    toolbar->addAction( &reloadAction() );
    toolbar->addAction( &abortAction() );

    // dock widgets
    placesPanel_ = new DockWidget( "Places", this, "PLACES" );
    placesPanel_->setAllowedAreas( Qt::LeftDockWidgetArea|Qt::RightDockWidgetArea );
    placesPanel_->setUseScrollArea( true );
    placesPanel_->visibilityAction().setText( "Show Favorite Places" );
    addDockWidget( Qt::LeftDockWidgetArea, placesPanel_ );

    placesWidget_ = new PlacesWidget();
    placesWidget_->setIconProvider( iconProvider_ );
    placesPanel_->setMainWidget( placesWidget_ );

    if( !placesPanel_->visibilityAction().isChecked() )
    { placesPanel_->hide(); }

    if( placesWidget_->isEmpty() )
    {
        FileInfo fileInfo( Util::home(), FileInfo::Folder );
        fileInfo.update();
        placesWidget_->add( "Home (local)", fileInfo );
    }

    // status bar
    setStatusBar( statusbar_ = new ProgressStatusBar( this ) );
    statusbar_->addClock();

    // menu bar
    Menu* menu( new Menu( this ) );
    setMenuBar( menu );
    connect( &menu->viewModeMenu(), SIGNAL( viewModeSelected( FileSelectionFrame::ViewMode ) ), SLOT( _updateViewMode( FileSelectionFrame::ViewMode ) ) );

    // interface connection
    connect( interface_, SIGNAL( contentAvailable( const FileInfo&, const FileInfo::List& ) ), SLOT( _updateContent( const FileInfo&, const FileInfo::List& ) ) );
    connect( interface_, SIGNAL( macroStarted( const FtpMacro& ) ), SLOT( _macroStarted( const FtpMacro& ) ) );
    connect( interface_, SIGNAL( macroCompleted( const FtpMacro&, bool ) ), SLOT( _macroCompleted( const FtpMacro&, bool ) ) );
    connect( interface_, SIGNAL( commandCompleted( const FtpCommand&, bool ) ), SLOT( _commandCompleted( const FtpCommand&, bool ) ) );
    connect( interface_, SIGNAL( fileInfoUpdated( const FileInfo& ) ), SLOT( _updateFileInfo( const FileInfo& ) ) );
    connect( interface_, SIGNAL( fileInfoUpdated( const FileInfo::List& ) ), SLOT( _updateFileInfo( const FileInfo::List& ) ) );

    connect( interface_, SIGNAL( maxProgressAvailable( int ) ), &statusbar_->progressBar(), SLOT( setMaximum( int ) ) );
    connect( interface_, SIGNAL( progressAvailable( int ) ), &statusbar_->progressBar(), SLOT( setValue( int ) ) );

    connect( interface_, SIGNAL( commandAdded( const FtpCommand& ) ), &commandDialog_->commandModel(), SLOT( add( const FtpCommand& ) ) );
    connect( interface_, SIGNAL( commandsAdded( const FtpCommand::List& ) ), &commandDialog_->commandModel(), SLOT( add( const FtpCommand::List& ) ) );
    connect( interface_, SIGNAL( commandCompleted( const FtpCommand&, bool ) ), &commandDialog_->commandModel(), SLOT( remove( const FtpCommand& ) ) );

    connect( interface_, SIGNAL( macroAdded( const FtpMacro& ) ), &commandDialog_->macroModel(), SLOT( add( const FtpMacro& ) ) );
    connect( interface_, SIGNAL( macroCompleted( const FtpMacro&, bool ) ), &commandDialog_->macroModel(), SLOT( remove( const FtpMacro& ) ) );

    connect( interface_, SIGNAL( busyStateChanged( bool ) ), SLOT( _updateBusyButtons( bool ) ) );
    connect( interface_, SIGNAL( busyStateChanged( bool ) ), &abortAction(), SLOT( setEnabled( bool ) ) );
    connect( &interface_->ftp(), SIGNAL( stateChanged( int ) ), this, SLOT( _stateChanged( int ) ) );
    connect( interface_, SIGNAL( logAvailable( QString, FORMAT::TextFormatFlags, QColor, int ) ), &logDialog().editor(), SLOT( append( QString, FORMAT::TextFormatFlags, QColor, int ) ) );

    connect( this, SIGNAL( macroRequested( const FtpMacro& ) ), interface_, SLOT( addMacro( const FtpMacro& ) ) );
    connect( this, SIGNAL( macrosRequested( const FtpMacro::List& ) ), interface_, SLOT( addMacros( const FtpMacro::List& ) ) );

    foreach( FileSelectionFrame* frame, selectionFrames() )
    {

        connect( frame, SIGNAL( fileSelected( const FileInfo& ) ), SLOT( _fileSelected( const FileInfo& ) ) );
        connect( frame, SIGNAL( actionsChanged( void ) ), SLOT( _updateNavigationActions( void ) ) );
        connect( frame, SIGNAL( clipboardChanged( const FileSelectionClipboard& ) ), SLOT( _updateClipboard( const FileSelectionClipboard& ) ) );
        connect( frame, SIGNAL( pasteClipboardRequested( const FileInfo& ) ), SLOT( _pasteClipboard( const FileInfo& ) ) );
        connect( frame, SIGNAL( openFilesRequested( const FileInfo::List& ) ), SLOT( _openFiles( const FileInfo::List& ) ) );

        connect( frame, SIGNAL( macroRequested( const FtpMacro& ) ), interface_, SLOT( addMacro( const FtpMacro& ) ) );
        connect( frame, SIGNAL( macrosRequested( const FtpMacro::List& ) ), interface_, SLOT( addMacros( const FtpMacro::List& ) ) );
        connect( frame, SIGNAL( requestFileInformation( const FileInfo&, const QIcon& ) ), SLOT( _fileInformation( const FileInfo&, const QIcon& ) ) );

        connect( &frame->model(), SIGNAL( transferRequested( const FileInfo&, const FileInfo::List& ) ), SLOT( _pasteSelection( const FileInfo&, const FileInfo::List& ) ) );
        connect( &frame->visibilityAction(), SIGNAL( toggled( bool ) ), SLOT( _updateVisibilityActions() ) );
        connect( frame->list().selectionModel(), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ), SLOT( _updateButtons() ) );
    }

    connect( &localSelectionFrame_->visibilityAction(), SIGNAL( toggled( bool ) ), SLOT( _updatePlacesItems( bool ) ) );

    // places
    connect( placesWidget_, SIGNAL( contentsChanged( void ) ), SLOT( _updatePlacesItems( void ) ) );
    connect( placesWidget_, SIGNAL( itemSelected( const BaseFileInfo& ) ), SLOT( _placeSelected( const BaseFileInfo& ) ) );

    // connections
    connect( &fileSystemWatcher_, SIGNAL( directoryChangedDelayed( const QString& ) ), &localSelectionFrame_->reloadAction(), SLOT( trigger( void ) ) );

    // active frame change
    connect( qApp, SIGNAL( focusChanged( QWidget*, QWidget* ) ), SLOT( _updateActiveFrame( QWidget*, QWidget* ) ) );

    connect( &showHiddenFilesAction(), SIGNAL( toggled( bool ) ), SLOT( _toggleShowHiddenFiles( bool ) ) );

    // configuration
    connect( &application, SIGNAL( configurationChanged( void ) ), SLOT( _updateConfiguration( void ) ) );

    // update buttons
    _updateConfiguration();
    _updateClipboard();
    _updateButtons();
    _updateBusyButtons( false );

    // initialize local home path
    localSelectionFrame_->setHomePath( Util::home() );

    // initialize local root path list
    File::List rootPathList;
    foreach( const QFileInfo& fileInfo, QDir::drives() )
    {
        Debug::Throw() << "MainWindow::MainWindow - root path: " << fileInfo.path() << endl;
        rootPathList << fileInfo.path();
    }

    localSelectionFrame_->setRootPathList( rootPathList );

    // initialize local path to working directory
    File path( Util::workingDirectory() );
    localSelectionFrame_->setPath( path );
    _fileSelected( FileInfo( path, FileInfo::Folder ) );

    // initialize remote home directory
    remoteSelectionFrame_->setHomePath( File( "/" ) );

    // reset remote file
    remoteSelectionFrame_->clear();
    remoteSelectionFrame_->setEnabled( false );

    // busy widget
    busyWidget_ = new BusyWidget( main, BusyWidget::BottomRight );
    busyWidget_->hide();

}

//____________________________________________________________________________________________
FileSelectionFrame& MainWindow::activeFrame( void ) const
{ return *(localSelectionFrame_->isActive() ? localSelectionFrame_:remoteSelectionFrame_); }

//____________________________________________________________________________________________
void MainWindow::connectToHost( QString host )
{
    Debug::Throw( "MainWindow::connectToHost.\n" );

    ConnectionAttributes attributes;

    // parse host and user
    int position(0);
    if( (position = host.indexOf( "@" )) >= 0 )
    {
        attributes.setUser( host.left( position ) );
        attributes.setHost( host.mid( position+1 ) );
    } else attributes.setHost( host );

    ConnectionDialog dialog( this );
    dialog.frame().load( attributes );
    dialog.centerOnParent();

    if( !dialog.centerOnParent().exec() ) return;

    // disconnect if already connected
    if( interface_->ftp().state() == QFtp::LoggedIn ) emit macroRequested( FtpMacro( FtpMacro::CloseConnection ) );

    // update attributes from dialog
    attributes_ = dialog.frame().attributes();

    // try connect
    emit macroRequested(
        FtpMacro( FtpMacro::Connect )
        << attributes_.host()
        << QString().setNum( attributes_.port() )
        << attributes_.user()
        << attributes_.password()
        );
}

//____________________________________
void MainWindow::closeEvent( QCloseEvent *event )
{
    Debug::Throw( "MainWindow::closeEvent.\n" );
    event->accept();
    qApp->quit();
}

//_______________________________________________________________________
bool MainWindow::_checkTransfer( const FileInfo& destination, const FileInfo::List& selection )
{

    Debug::Throw()
        << "MainWindow::_checkTransfer -"
        << " destination: " << destination.file()
        << " path: " << selectionFrame( destination.location() ).path().file()
        << endl;

    // make sure we are not copying file to same location
    if( destination.file() == selectionFrame( destination.location() ).path().file() )
    {

        foreach( const FileInfo& fileInfo, selection )
        {

            QString fileInfoPath( fileInfo.path() );
            if( fileInfoPath.endsWith('/') ) fileInfoPath = fileInfoPath.left( fileInfoPath.size()-1 );
            if( fileInfo.location() == destination.location() && fileInfoPath == destination.file() )
            {

                InformationDialog dialog( this, "Items cannot be copied onto themselves. <copy> aborted." );
                dialog.setWindowTitle( "Warning - Gridftp" );
                dialog.exec();
                return false;
            }
        }

    }

    // make sure one does not copy one file into itself
    foreach( const FileInfo& fileInfo, selection )
    {
        if(
            destination.isFolder() &&
            fileInfo.location() == destination.location() &&
            fileInfo.file() == destination.file() )
        {

            InformationDialog dialog( this, "A folder cannot be copied into itself. <copy> aborted." );
            dialog.setWindowTitle( "Warning - Gridftp" );
            dialog.exec();
            return false;

        }

    }

    return true;

}

//____________________________________________________________________________________________
void MainWindow::_paste( const FileInfo& destination, const FileInfo::List& selection )
{

    Debug::Throw( "MainWindow::_paste.\n" );

    // retrieve selected files
    if( selection.empty() ) return;

    // check if copu is local to local
    const FileInfo sourcePath( selection.front().path(), FileInfo::Folder|selection.front().location() );
    const bool localOnly( sourcePath.isLocal() && destination.isLocal() );

    // check transfer types
    // allowed are local to local, local to remote and remote to local
    if( !( localOnly || interface_->ftp().state() == QFtp::LoggedIn ) ) return;

    // add list recursive commands
    FtpMacro::List macros;
    macros << FtpMacro::Clear;
    foreach( const FileInfo& fileInfo, selection )
    { macros << ( FtpMacro( FtpMacro::ListRecursive ) << fileInfo << destination ); }

    // check conflicts, copy, and check result
    macros << ( FtpMacro( FtpMacro::CheckConflicts ) << sourcePath << destination );
    macros << ( FtpMacro( FtpMacro::CopyRecursive ) << sourcePath << destination );
    macros << ( FtpMacro( FtpMacro::CheckFilesCopy ) << sourcePath << destination );

    // update destination
    macros << ( FtpMacro( FtpMacro::ClearSelection ) << destination );

    macros << FtpMacro::Clear;
    macros << ( FtpMacro( FtpMacro::ShowDirectory ) << selectionFrame( destination.type() ).path() );

    foreach( const FileInfo& fileInfo, _destinationList( destination, selection ) )
    {  macros << ( FtpMacro( FtpMacro::Select ) << fileInfo ); }

    emit macrosRequested( macros );

}

//_______________________________________________________________________
FileSelectionClipboard MainWindow::_move( const FileInfo& destination, const FileInfo::List& selection )
{

    Debug::Throw( "MainWindow::_move.\n" );

    // list of macros
    FtpMacro::List macros;

    // for local-to-local or remote-to-remote move, use Rename command
    if( selection.front().location() == destination.location() )
    {

        // loop over files in clipboard
        foreach( const FileInfo& fileInfo, selection )
        {

            // generate destination file
            FileInfo destinationFile( fileInfo );
            destinationFile.setFile( fileInfo.file().localName().addPath( destination.file() ) );
            if( destination.isLocal() ) destinationFile.setLocal();
            else destinationFile.setRemote();

            // add macro
            macros << ( FtpMacro( FtpMacro::Rename ) << fileInfo << destinationFile );

        }

    } else {

        macros << FtpMacro::Clear;
        foreach( const FileInfo& fileInfo, selection )
        { macros << ( FtpMacro( FtpMacro::ListRecursive ) << fileInfo << destination ); }

        // emit copy command
        const FileInfo sourcePath( selection.front().path(), FileInfo::Folder|selection.front().location() );

        // generate checkConflicts, copy and checkFiles macros
        macros << ( FtpMacro( FtpMacro::CheckConflicts ) << sourcePath << destination );
        macros << ( FtpMacro( FtpMacro::CopyRecursive ) << sourcePath << destination );
        macros << ( FtpMacro( FtpMacro::CheckFilesMoved ) << sourcePath << destination );

        // generate remove macro
        // TODO: make sure it does not remove files that have not been properly copied
        macros << ( FtpMacro( FtpMacro::Remove ) << sourcePath );

    }

    // generate updated clipboard
    FileSelectionClipboard localClipboard( _destinationList( destination, selection ) );
    localClipboard.setMode( FileSelectionClipboard::Copy );

    // trigger destination update
    if( destination == selectionFrame( destination.type() ).path() )
    {
        macros << FtpMacro::Clear;
        macros << ( FtpMacro( FtpMacro::ShowDirectory ) << destination );

        foreach( const FileInfo& fileInfo, localClipboard )
        { macros << ( FtpMacro( FtpMacro::Select ) << fileInfo ); }

    }

    // decide if source has to be updated at the end of the macro
    const FileInfo sourcePath( selectionFrame( selection.front().type() ).path() );
    foreach( const FileInfo& fileInfo, selection )
    {
        if( fileInfo.path() == sourcePath.file() )
        {
            macros << FtpMacro::Clear;
            macros << ( FtpMacro( FtpMacro::ShowDirectory ) << sourcePath );
            break;
        }
    }

    // emit macros
    if( !macros.empty() ) emit macrosRequested( macros );

    // return local clipboard, for replacement
    return localClipboard;
}

//_______________________________________________________________________
FileInfo::List MainWindow::_destinationList( const FileInfo& destination, const FileInfo::List& selection ) const
{

    FileInfo::List destinationList;
    foreach( const FileInfo& fileInfo, selection )
    {

        // generate destination file
        FileInfo destinationFile( fileInfo );
        destinationFile.setFile( fileInfo.file().localName().addPath( destination.file() ) );
        if( destination.isLocal() ) destinationFile.setLocal();
        else destinationFile.setRemote();
        destinationList << destinationFile;

    }

    return destinationList;

}

//___________________________________________________________________________
void MainWindow::_updatePath( const FileInfo& fileInfo )
{

    Debug::Throw() << "MainWindow::_updatePath - file: " << fileInfo << endl;
    Q_ASSERT( fileInfo.isFolder() );

    // update path
    if( fileInfo.isLocal() || ( fileInfo.isRemote() && interface_->ftp().state() == QFtp::LoggedIn ) )
    {

        FtpMacro::List macros;
        macros
            << FtpMacro::Clear
            << ( FtpMacro( FtpMacro::ShowDirectory ) << fileInfo );

        emit macrosRequested( macros );

    }

    // also update fileSystemWatcher
    if( fileInfo.isLocal() )
    {
        const QStringList directories( fileSystemWatcher_.directories() );
        if( !directories.isEmpty() ) fileSystemWatcher_.removePaths( directories );
        if( !fileInfo.file().isEmpty() ) fileSystemWatcher_.addPath( fileInfo.file() );
    }

}

//______________________________________________________
void MainWindow::_updateWindowTitle( void )
{

    File path( activeFrame().path().file() );
    if( path.endsWith( '/' ) ) path = path.left( path.size()-1 );

    QString buffer;
    QTextStream what( &buffer );
    if( !path.isEmpty() ) what << path.localName() << " - ";
    what << "Qftp";
    setWindowTitle( buffer );

}

//______________________________________________________
void MainWindow::_updateVisibilityActions( void )
{

    Debug::Throw( "MainWindow::_updateVisibilityActions.\n" );
    localSelectionFrame_->visibilityAction().setEnabled( remoteSelectionFrame_->visibilityAction().isChecked() );
    remoteSelectionFrame_->visibilityAction().setEnabled( localSelectionFrame_->visibilityAction().isChecked() );

    // handle focus
    if( !localSelectionFrame_->visibilityAction().isChecked() ) remoteSelectionFrame_->setFocus();
    if( !remoteSelectionFrame_->visibilityAction().isChecked() ) localSelectionFrame_->setFocus();

}

//______________________________________________________
void MainWindow::_updateNavigationActions( void )
{
    Debug::Throw( "MainWindow::_updateNavigationActions.\n" );
    reloadAction().setEnabled( activeFrame().reloadAction().isEnabled() );
    previousDirectoryAction().setEnabled( activeFrame().previousDirectoryAction().isEnabled() );
    nextDirectoryAction().setEnabled( activeFrame().nextDirectoryAction().isEnabled() );
    parentDirectoryAction().setEnabled( activeFrame().parentDirectoryAction().isEnabled() );
    homeDirectoryAction().setEnabled( activeFrame().homeDirectoryAction().isEnabled() );
    return;
}

//____________________________________________________________________________________________
void MainWindow::_updateActiveFrame( QWidget*, QWidget* widget )
{

    Debug::Throw() << "MainWindow::_updateActiveFrame -"
        << " widget: " << widget
        << " (" << (widget ? widget->metaObject()->className():"none") << ")"
        << endl;

    if( localSelectionFrame_->owns( widget ) )
    {

        localSelectionFrame_->setActive( true );
        remoteSelectionFrame_->setActive( false );
        _updateNavigationActions();
        _updateWindowTitle();

    } else if(  remoteSelectionFrame_->owns( widget ) ) {

        localSelectionFrame_->setActive( false );
        remoteSelectionFrame_->setActive( true );
        _updateNavigationActions();
        _updateWindowTitle();

    }

}

//____________________________________________________________________________________________
void MainWindow::_updateViewMode( FileSelectionFrame::ViewMode mode )
{
    FileSelectionFrame& selectionFrame( activeFrame() );

    selectionFrame.setViewMode( mode );
    selectionFrame.setShowNavigator( showNavigator_ && mode == FileSelectionFrame::LIST );

}

//________________________________________________________________________
void MainWindow::_updatePlacesItems( void )
{ _updatePlacesItems( localSelectionFrame_->visibilityAction().isChecked() ); }

//________________________________________________________________________
void MainWindow::_updatePlacesItems( bool value )
{

    Debug::Throw( "MainWindow::_updatePlacesItems.\n" );
    foreach( const BaseFileInfo& fileInfo, placesWidget_->items() )
    {
        if( fileInfo.isLocal() )
        { placesWidget_->setItemIsValid( fileInfo, value && fileInfo.file().exists() ); }
    }

}

//________________________________________________________________________
void MainWindow::_toggleShowHiddenFiles( bool value )
{
    Debug::Throw() << "MainWindow::_toggleShowHiddenFiles - " << value << endl;
    interface_->setShowHiddenFiles( value );
    localSelectionFrame_->setShowHiddenFiles( value );
    remoteSelectionFrame_->setShowHiddenFiles( value );
    XmlOptions::get().set<bool>( "SHOW_HIDDEN_FILES", value );
}

//____________________________________________________________________________________________
void MainWindow::_toggleConnection( bool state )
{

    Debug::Throw() << "MainWindow::_toggleConnection - state: " << state << endl;

    if( !enabled_ ) return;

    if( interface_->ftp().state() == QFtp::LoggedIn ) emit macroRequested( FtpMacro( FtpMacro::CloseConnection ) );
    else {

        ConnectionDialog dialog( this );
        dialog.setWindowTitle( "Connections - Qftp" );
        dialog.centerOnParent();
        if( !dialog.centerOnParent().exec() )
        {
            enabled_ = false;
            connectAction().setChecked( false );
            enabled_ = true;
            return;
        }

        attributes_ = dialog.frame().attributes();

        emit macroRequested(
            FtpMacro( FtpMacro::Connect )
            << attributes_.host()
            << QString().setNum( attributes_.port() )
            << attributes_.user()
            << attributes_.password()
            );

    }

}

//___________________________________________________________________________
void MainWindow::_abort( void )
{

    Debug::Throw( "MainWindow::_abort.\n" );

    // retrieve current local and remote path
    FileInfo localPath( localSelectionFrame_->path() );
    FileInfo remotePath( remoteSelectionFrame_->path() );

    // process Abort command
    interface_->clear();
    FtpMacro::List macros;
    macros
        << FtpMacro::Abort
        << ( FtpMacro( FtpMacro::ShowDirectory ) << localPath )
        << ( FtpMacro( FtpMacro::ShowDirectory ) << remotePath );

    emit macrosRequested( macros );

    return;

}

//____________________________________________________________________________________________
void MainWindow::_fileSelected( const FileInfo& fileInfo )
{
    Debug::Throw() << "MainWindow::_fileSelected - file: " << fileInfo << endl;
    if( fileInfo.isFolder() ) _updatePath( fileInfo );
    else  _openFiles( FileInfo::List() << fileInfo );
}

//____________________________________________________________________________________________
void MainWindow::_placeSelected( const BaseFileInfo& fileInfo )
{
    Debug::Throw() << "MainWindow::_placeSelected - file: " << fileInfo << endl;
    if( !fileInfo.isFolder() ) return;

    // update path
    _updatePath( fileInfo );

    // set focus to correct selection frame
    selectionFrame( fileInfo.type() ).setFocus();

    return;
}

//_______________________________________________________________________
void MainWindow::_reload( void )
{ if( activeFrame().reloadAction().isEnabled() ) activeFrame().reloadAction().trigger(); }

//_______________________________________________________________________
void MainWindow::_previousDirectory( void )
{ if( activeFrame().previousDirectoryAction().isEnabled() ) activeFrame().previousDirectoryAction().trigger(); }

//_______________________________________________________________________
void MainWindow::_nextDirectory( void )
{ if( activeFrame().nextDirectoryAction().isEnabled() ) activeFrame().nextDirectoryAction().trigger(); }

//_______________________________________________________________________
void MainWindow::_parentDirectory( void )
{ if( activeFrame().parentDirectoryAction().isEnabled() ) activeFrame().parentDirectoryAction().trigger(); }

//_______________________________________________________________________
void MainWindow::_homeDirectory( void )
{ if( activeFrame().homeDirectoryAction().isEnabled() ) activeFrame().homeDirectoryAction().trigger(); }


//______________________________________________________________
void MainWindow::_updateClipboard( const FileSelectionClipboard& selection )
{

    Debug::Throw( "MainWindow::_updateClipboard.\n" );
    clipboard_ = selection;

    {
        // for local frame one allow copy from either local or remote
        const bool enabled( !clipboard_.isEmpty() );
        localSelectionFrame_->pasteAction().setEnabled( enabled );
        localSelectionFrame_->pasteInFolderAction().setEnabled( enabled );

        if( enabled && !clipboard_.empty() ) localSelectionFrame_->pasteAction().setText( QString( "Paste " ) + clipboard_.description() );
        else localSelectionFrame_->pasteAction().setText( "Paste" );
    }

    {
        const bool connected( interface_->ftp().state() == QFtp::LoggedIn );
        const bool enabled( connected && !clipboard_.isEmpty() && (clipboard_.front().isLocal() || clipboard_.mode() == FileSelectionClipboard::Cut ) );
        remoteSelectionFrame_->pasteAction().setEnabled( enabled );
        remoteSelectionFrame_->pasteInFolderAction().setEnabled( enabled );

        if( enabled && !clipboard_.empty() ) remoteSelectionFrame_->pasteAction().setText( QString( "Paste " ) + clipboard_.description() );
        else remoteSelectionFrame_->pasteAction().setText( "Paste" );
    }
}

//_______________________________________________________________________
void MainWindow::_pasteSelection( const FileInfo& constDestination, const FileInfo::List& selection )
{

    Debug::Throw( "MainWindow::_pasteSelection.\n" );
    if( selection.empty() ) return;

    // make sure valid destination is set
    FileInfo destination( constDestination );
    if( destination.file().isEmpty() ) destination = selectionFrame( destination.location() ).path();

    // check transfer
    if( !_checkTransfer( destination, selection ) ) return;

    // disable remote-to-remote copy (not supported by qft)
    menuActions_[0]->setEnabled( !( destination.isRemote() && selection.front().isRemote() ) );

    // open menu
    QAction* action( QMenu::exec( menuActions_, QCursor::pos() ) );
    if( action == menuActions_[0] ) _paste( destination, selection );
    else if( action == menuActions_[1] ) _move( destination, selection );
    else return;

}

//_______________________________________________________________________
void MainWindow::_pasteClipboard( const FileInfo& destination )
{

    Debug::Throw( "MainWindow::_pasteClipboard.\n" );
    if( clipboard_.empty() ) return;
    if( !_checkTransfer( destination, clipboard_ ) ) return;

    // perform command
    if( clipboard_.mode() == FileSelectionClipboard::Copy ) _paste( destination, clipboard_ );
    else clipboard_  = _move( destination, clipboard_ );

}

//______________________________________________________________
void MainWindow::_openFiles( const FileInfo::List& files )
{
    Debug::Throw( "MainWindow::_openFiles.\n" );

    // ask confirmation
    OpenFilesDialog dialog( this, files );
    dialog.setWindowTitle( "Open Files - Qftp" );

    // retrieve applications from options
    foreach( const Option& application, XmlOptions::get().specialOptions( "APPLICATIONS" ) )
    { dialog.actionComboBox().addItem( application.raw() ); }

    if( !dialog.exec() ) return;

    // retrieve command
    QString command( dialog.actionComboBox().currentText() );
    if( command.isEmpty() )
    {
        InformationDialog( this, "No command specified to open the selected files. <Open> canceled." ).exec();
        return;
    }

    // update options
    XmlOptions::get().add( "APPLICATIONS", Option( command, Option::Recordable|Option::Current ) );

    // retrieve local and remote path
    FileInfo localPath( localSelectionFrame_->path() );
    FileInfo remotePath( remoteSelectionFrame_->path() );

    // create list of macros
    FtpMacro::List macros;
    macros << FtpMacro::Clear;

    // loop over selected items and add to interface
    foreach( const FileInfo& fileInfo, files )
    { macros << ( FtpMacro( FtpMacro::Add ) << fileInfo ); }

    // submit open command
    macros << ( FtpMacro( FtpMacro::Open ) << localPath << remotePath << command );

    // emit
    emit macrosRequested( macros );

}

//____________________________________________________________________________________________
void MainWindow::_macroStarted( const FtpMacro& macro )
{
    Debug::Throw( "MainWindow::_macroStarted.\n" );
    switch( macro.type() )
    {
        case FtpMacro::Remove:
        case FtpMacro::Open:
        case FtpMacro::CopyRecursive:
        statusbar_->showProgressBar();
        break;

        default: break;
    }

}

//____________________________________________________________________________________________
void MainWindow::_macroCompleted( const FtpMacro& macro, bool )
{
    Debug::Throw( "MainWindow::_macroCompleted.\n" );

    switch( macro.type() )
    {

        case FtpMacro::Remove:
        case FtpMacro::Open:
        statusbar_->hideProgressBar();
        break;

        case FtpMacro::CopyRecursive:
        {

            statusbar_->hideProgressBar();

            // adjust column sizes
            if( AnimatedTreeView* treeView = qobject_cast<AnimatedTreeView*>( &selectionFrame( FileInfo( macro.argument(0) ).type() ).list() ) )
            { treeView->resizeColumnToContents( FileInfoModel::FILE ); }
            break;

        }

        case FtpMacro::CheckConflicts:
        {

            // retrieve files and check if there is a conflict
            FileInfo::List& files( interface_->filesRecursive() );
            if( !std::count_if( files.begin(), files.end(), FileInfo::ConflictFTor() ) ) break;


            // create temporary list
            FileInfo::List local;
            foreach( const FileInfo& fileInfo, files )
            { if( fileInfo.isDocument() && fileInfo.isConflict() ) local << fileInfo; }

            // create dialog
            ConflictDialog dialog( this );
            dialog.setWindowTitle( "Conflicts - Qftp" );
            dialog.setFiles(
                macro.argument(0).file(),
                macro.argument(1).file(),
                local );

            if( dialog.centerOnParent().exec() == QDialog::Rejected ) files.clear();
            else dialog.updateFiles( files );
            break;

        }

        case FtpMacro::CheckFilesCopy:
        case FtpMacro::CheckFilesMoved:
        {

            // retrieve files and check if copy has succedded
            const FileInfo::List& files( interface_->filesRecursive() );

            FileInfo::List local;
            foreach( const FileInfo& fileInfo, files )
            { if( fileInfo.isFailed() ) local << fileInfo; }

            // check if there are some failed files
            if( local.empty() ) break;

            // create dialog
            const File sourcePath( macro.argument(0).file() );
            const File destinationPath( macro.argument(1).file() );

            CheckFilesDialog dialog( this );
            dialog.setWindowTitle( "Error - Qftp" );
            dialog.setFiles(
                sourcePath,
                destinationPath,
                local );

            // this needs double check. Probably broken
            if( dialog.centerOnParent().exec() == QDialog::Accepted )
            {

                // get selection
                if( !(local = dialog.selectedFiles()).empty() )
                {
                    if( macro.type() == FtpMacro::CheckFilesCopy ) _paste( destinationPath, local );
                    else _move( destinationPath, local );
                }

            }
            break;

        }

        case FtpMacro::ClearSelection:
        selectionFrame( macro.argument(0).type() ).clearSelection();
        break;

        case FtpMacro::Select:
        {
            const FileInfo fileInfo( macro.argument(0) );
            selectionFrame( fileInfo.type() ).select( fileInfo );
            break;
        }


        default:
        break;

    }

    return;

}

//____________________________________________________________________________________________
void MainWindow::_commandCompleted( const FtpCommand& command, bool )
{

    switch( command.type() )
    {

        case FtpCommand::Copy:
        {
            // retrieve fileInfo from command
            // and update the corresponding list
            FileInfo fileInfo( command.argument(1) );
            selectionFrame( fileInfo.type() ).add( fileInfo );
        }
        break;

        case FtpCommand::Remove:
        {
            FileInfo fileInfo( command.argument(0) );
            if( fileInfo.isLocal() ) localSelectionFrame_->remove( fileInfo );
            if( fileInfo.isRemote() ) remoteSelectionFrame_->remove( fileInfo );
        }
        break;

        case FtpCommand::CreateDirectory:
        {

            FileInfo fileInfo( command.argument(0) );
            if( fileInfo.isLocal() ) localSelectionFrame_->add( fileInfo );
            if( fileInfo.isRemote() ) remoteSelectionFrame_->add( fileInfo );

        }
        break;


        default:
        break;
    }
}

//___________________________________________________________________________
void MainWindow::_updateContent( const FileInfo& path, const FileInfo::List& files )
{

    FileSelectionFrame& selectionFrame( this->selectionFrame( path.type() ) );

    // build file info list with local filenames
    FileInfo::List local;
    foreach( const FileInfo& fileInfo, files )
    { if( fileInfo.isLocal() == path.isLocal() ) local << fileInfo; }

    // clear the model if path differs from selectionFrame path
    if( path.file() != selectionFrame.path().file() )
    {

        selectionFrame.setPath( path.file() );
        selectionFrame.model().clear();
        selectionFrame.model().set( local );
        _updateWindowTitle();

    } else {

        selectionFrame.model().update( local );

    }

    // navigator
    selectionFrame.setShowNavigator( showNavigator_ && selectionFrame.viewMode() == FileSelectionFrame::LIST );

}

//___________________________________________________________________________
void MainWindow::_updateFileInfo( const FileInfo& info )
{
    Debug::Throw() << "MainWindow::_updateFileInfo - file: " << info.file() << (info.isRemote() ? " (remote)":" (local)") << endl;
    selectionFrame( info.type() ).add( info );
}

//___________________________________________________________________________
void MainWindow::_stateChanged( int state )
{

    Debug::Throw( "MainWindow::_stateChanged.\n" );

    enabled_ = false;
    QString buffer;
    QTextStream what( &buffer );
    switch( state )
    {

        case QFtp::Unconnected:
        {
            remoteSelectionFrame_->clear();
            connectAction().setChecked( false );
            _updateButtons();
            remoteSelectionFrame_->setEnabled( false );
            what << "not connected";
        }
        break;

        case QFtp::HostLookup:
        { what << "Looking for host " << attributes_.host(); }
        break;

        case QFtp::Connecting:
        { what << "Connecting"; }
        break;

        case QFtp::Connected:
        { what << "Connected"; }
        break;

        case QFtp::LoggedIn:
        {
            remoteSelectionFrame_->setPath( File("/") );
            _fileSelected( FileInfo( "/", FileInfo::Folder|FileInfo::Remote ) );
            _updateButtons();
            remoteSelectionFrame_->setEnabled( true );
            connectAction().setChecked( true );
            what << "Logged in as " << attributes_.user();
        }
        break;

        default:
        break;

    }

    enabled_ = true;
    if( buffer.isEmpty() ) return;

    //! retrieve StateFrame
    statusbar_->label(0).setText( buffer, true );

}

//___________________________________________________________________________
void MainWindow::_updateButtons( void )
{

    Debug::Throw( "MainWindow::_updateButtons.\n" );
    const bool connected( interface_->ftp().state() == QFtp::LoggedIn );
    remoteSelectionFrame_->homeDirectoryAction().setEnabled( connected );
    remoteSelectionFrame_->reloadAction().setEnabled( connected );
    remoteSelectionFrame_->newFolderAction().setEnabled( connected );

    if( !connected )
    {
        remoteSelectionFrame_->previousDirectoryAction().setEnabled( false );
        remoteSelectionFrame_->nextDirectoryAction().setEnabled( false );
        remoteSelectionFrame_->parentDirectoryAction().setEnabled( false );
        remoteSelectionFrame_->renameAction().setEnabled( false );
        remoteSelectionFrame_->removeAction().setEnabled( false );
        remoteSelectionFrame_->openAction().setEnabled( false );
    }

    _updateClipboard( clipboard_ );

}

//___________________________________________________________________________
void MainWindow::_updateBusyButtons( bool state )
{

    Debug::Throw( "MainWindow::_updateBusyButtons.\n" );
    abortAction().setEnabled( state );

}

//__________________________________________________________________________
void MainWindow::_showCommandDialog( void )
{

    Debug::Throw( "MainWindow::_showCommandDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", true );
    commandDialog_->show();
    commandDialog_->raise();

}

//_______________________________________________
void MainWindow::_commandDialogHidden( void )
{
    Debug::Throw( "Application::_commandDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_COMMAND_LIST", false );
}

//__________________________________________________________________________
void MainWindow::_showLogDialog( void )
{

    Debug::Throw( "MainWindow::_showLogDialog.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", true );
    logDialog().show();
    logDialog().raise();

}

//_______________________________________________
void MainWindow::_logDialogHidden( void )
{
    Debug::Throw( "MainWindow::_logDialogHidden.\n" );
    XmlOptions::get().set<bool>( "SHOW_LOG", false );
}

//____________________________________________________________________________________________
void MainWindow::_toggleShowNavigator( bool value )
{

    Debug::Throw() << "MainWindow::_toggleShowNavigator - " << value << endl;
    if( showNavigator_ == value ) return;
    showNavigator_ = value;
    foreach( FileSelectionFrame* selectionFrame, selectionFrames() )
    { selectionFrame->setShowNavigator( showNavigator_ && selectionFrame->viewMode() == FileSelectionFrame::LIST ); }

}

//_______________________________________________
void MainWindow::_fileInformation( const FileInfo& fileInfo, const QIcon& icon )
{

    FileInformationDialog dialog( this, fileInfo );
    dialog.setIcon( icon );

    connect( &dialog, SIGNAL( macrosRequested( const FtpMacro::List& ) ), interface_, SLOT( addMacros( const FtpMacro::List& ) ) );
    connect( interface_, SIGNAL( sizeAvailable( const FileInfo&, qint64 ) ), &dialog, SLOT( updateSize( const FileInfo&, qint64 ) ) );
    connect( interface_, SIGNAL( macroStarted( const FtpMacro& ) ), &dialog, SLOT( macroStarted( const FtpMacro& ) ) );
    connect( interface_, SIGNAL( macroCompleted( const FtpMacro&, bool ) ), &dialog, SLOT( macroCompleted( const FtpMacro& ) ) );

    dialog.exec();

}

//____________________________________________________________________________________________
void MainWindow::_updateConfiguration( void )
{
    Debug::Throw( "MainWindow::_updateConfiguration.\n" );

    // command dialog
    if( XmlOptions::get().get<bool>( "SHOW_COMMAND_LIST" ) ) _showCommandDialog();
    else commandDialog_->hide();

    // log dialog
    if( XmlOptions::get().get<bool>( "SHOW_LOG" ) ) _showLogDialog();
    else logDialog().hide();

    // navigator
    _toggleShowNavigator( XmlOptions::get().get<bool>( "SHOW_NAVIGATOR" ) );

    // hidden files
    showHiddenFilesAction_->setChecked( XmlOptions::get().get<bool>( "SHOW_HIDDEN_FILES" ) );

    // clear cached icons
    iconProvider_->clear();

    // update places items
    _updatePlacesItems( localSelectionFrame_->visibilityAction().isChecked() );

}

//_______________________________________________________________________
void MainWindow::_installActions( void )
{
    Debug::Throw( "MainWindow::_installActions.\n" );

    // connect
    addAction( connectAction_ = new QAction( IconEngine::get( ICONS::CONNECT ), "Connect", this ) );
    connectAction_->setToolTip( "Toggle connection" );
    connectAction_->setCheckable( true );
    connectAction_->setChecked( false );
    connect( connectAction_, SIGNAL( toggled( bool ) ), SLOT( _toggleConnection( bool ) ) );

    // home directory
    addAction( homeDirectoryAction_ = new QAction( IconEngine::get( ICONS::HOME ), "Home", this ) );
    connect( homeDirectoryAction_, SIGNAL( triggered() ), SLOT( _homeDirectory() ) );
    homeDirectoryAction_->setToolTip( "Change path to user home" );

    // previous directory
    addAction( previousDirectoryAction_ = new QAction( IconEngine::get( ICONS::PREVIOUS_DIRECTORY ), "Previous", this ) );
    connect( previousDirectoryAction_, SIGNAL( triggered() ), SLOT( _previousDirectory() ) );
    previousDirectoryAction_->setToolTip( "Change path to previous directory (from history)" );

    // next directory (from history)
    addAction( nextDirectoryAction_ = new QAction( IconEngine::get( ICONS::NEXT_DIRECTORY ), "Next", this ) );
    connect( nextDirectoryAction_, SIGNAL( triggered() ), SLOT( _nextDirectory() ) );
    nextDirectoryAction_->setToolTip( "Change path to next directory (from history)" );

    // parent directory in tree
    addAction( parentDirectoryAction_ = new QAction( IconEngine::get( ICONS::PARENT ), "Parent Directory", this ) );
    connect( parentDirectoryAction_, SIGNAL( triggered() ), SLOT( _parentDirectory() ) );
    parentDirectoryAction_->setToolTip( "Change path to parent directory" );

    // reload
    addAction( reloadAction_ = new QAction( IconEngine::get( ICONS::RELOAD ), "Reload", this ) );
    connect( reloadAction_, SIGNAL( triggered() ), SLOT( _reload() ) );
    reloadAction_->setShortcut( QKeySequence::Refresh );
    reloadAction_->setToolTip( "Reload" );

    // abort
    addAction( abortAction_ = new QAction( IconEngine::get( ICONS::ABORT ), "Abort", this ) );
    abortAction_->setToolTip( "Abort transfer" );
    connect( abortAction_, SIGNAL( triggered() ), SLOT( _abort() ) );

    // show command
    addAction( showCommandDialogAction_ = new QAction( IconEngine::get( ICONS::FIND ), "Show Command List", this ) );
    connect( showCommandDialogAction_, SIGNAL( triggered( void ) ), SLOT( _showCommandDialog( void ) ) );

    // show log
    addAction( showLogDialogAction_ = new QAction( IconEngine::get( ICONS::FIND ), "Show Log", this ) );
    connect( showLogDialogAction_, SIGNAL( triggered( void ) ), SLOT( _showLogDialog( void ) ) );

    // show hidden files
    addAction( showHiddenFilesAction_ = new QAction( "Show Hidden Files", this ) );
    showHiddenFilesAction_->setCheckable( true );
    showHiddenFilesAction_->setChecked( false );

    // menu actions
    QAction* action;
    menuActions_ << ( action = new QAction( IconEngine::get( ICONS::COPY ), "Copy Here", this ) );
    menuActions_ << ( action = new QAction( IconEngine::get( ICONS::MOVE ), "Move Here", this ) );
    menuActions_ << ( action = new QAction( this ) ); action->setSeparator( true );
    menuActions_ << ( action = new QAction( IconEngine::get( ICONS::DIALOG_CANCEL ), "Cancel", this ) );
    action->setShortcut( Qt::Key_Escape );
}
